# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/vectors/v1/filters.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dqwak/vectors/v1/filters.proto\x12\x1cqwak.vector.store.serving.v1\x1a\x1fgoogle/protobuf/timestamp.proto\"\xbb\x01\n\rAtomicLiteral\x12\x18\n\x0estring_literal\x18\x01 \x01(\tH\x00\x12\x15\n\x0bint_literal\x18\x02 \x01(\x03H\x00\x12\x16\n\x0c\x62ool_literal\x18\x03 \x01(\x08H\x00\x12\x18\n\x0e\x64ouble_literal\x18\x04 \x01(\x01H\x00\x12\x37\n\x11timestamp_literal\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00\x42\x0e\n\x0cliteral_type\"\xf9\x04\n\x06\x46ilter\x12.\n\x02or\x18\x01 \x01(\x0b\x32 .qwak.vector.store.serving.v1.OrH\x00\x12\x30\n\x03\x61nd\x18\x02 \x01(\x0b\x32!.qwak.vector.store.serving.v1.AndH\x00\x12\x31\n\x02\x65q\x18\x03 \x01(\x0b\x32#.qwak.vector.store.serving.v1.EqualH\x00\x12\x34\n\x02ne\x18\x04 \x01(\x0b\x32&.qwak.vector.store.serving.v1.NotEqualH\x00\x12\x37\n\x02gt\x18\x05 \x01(\x0b\x32).qwak.vector.store.serving.v1.GreaterThanH\x00\x12=\n\x03gte\x18\x06 \x01(\x0b\x32..qwak.vector.store.serving.v1.GreaterThanEqualH\x00\x12\x34\n\x02lt\x18\x07 \x01(\x0b\x32&.qwak.vector.store.serving.v1.LessThanH\x00\x12:\n\x03lte\x18\x08 \x01(\x0b\x32+.qwak.vector.store.serving.v1.LessThanEqualH\x00\x12\x32\n\x04like\x18\t \x01(\x0b\x32\".qwak.vector.store.serving.v1.LikeH\x00\x12\x37\n\x07is_null\x18\n \x01(\x0b\x32$.qwak.vector.store.serving.v1.IsNullH\x00\x12>\n\x0bis_not_null\x18\x0b \x01(\x0b\x32\'.qwak.vector.store.serving.v1.IsNotNullH\x00\x42\r\n\x0b\x66ilter_case\"m\n\x02Or\x12\x32\n\x04left\x18\x01 \x01(\x0b\x32$.qwak.vector.store.serving.v1.Filter\x12\x33\n\x05right\x18\x02 \x01(\x0b\x32$.qwak.vector.store.serving.v1.Filter\"n\n\x03\x41nd\x12\x32\n\x04left\x18\x01 \x01(\x0b\x32$.qwak.vector.store.serving.v1.Filter\x12\x33\n\x05right\x18\x02 \x01(\x0b\x32$.qwak.vector.store.serving.v1.Filter\"U\n\x05\x45qual\x12\x10\n\x08property\x18\x01 \x01(\t\x12:\n\x05value\x18\x02 \x01(\x0b\x32+.qwak.vector.store.serving.v1.AtomicLiteral\"X\n\x08NotEqual\x12\x10\n\x08property\x18\x01 \x01(\t\x12:\n\x05value\x18\x02 \x01(\x0b\x32+.qwak.vector.store.serving.v1.AtomicLiteral\"[\n\x0bGreaterThan\x12\x10\n\x08property\x18\x01 \x01(\t\x12:\n\x05value\x18\x02 \x01(\x0b\x32+.qwak.vector.store.serving.v1.AtomicLiteral\"`\n\x10GreaterThanEqual\x12\x10\n\x08property\x18\x01 \x01(\t\x12:\n\x05value\x18\x02 \x01(\x0b\x32+.qwak.vector.store.serving.v1.AtomicLiteral\"X\n\x08LessThan\x12\x10\n\x08property\x18\x01 \x01(\t\x12:\n\x05value\x18\x02 \x01(\x0b\x32+.qwak.vector.store.serving.v1.AtomicLiteral\"]\n\rLessThanEqual\x12\x10\n\x08property\x18\x01 \x01(\t\x12:\n\x05value\x18\x02 \x01(\x0b\x32+.qwak.vector.store.serving.v1.AtomicLiteral\")\n\x04Like\x12\x10\n\x08property\x18\x01 \x01(\t\x12\x0f\n\x07pattern\x18\x02 \x01(\t\"\x1a\n\x06IsNull\x12\x10\n\x08property\x18\x01 \x01(\t\"\x1d\n\tIsNotNull\x12\x10\n\x08property\x18\x01 \x01(\tB\x1e\n\x1a\x63om.qwak.ai.vectors.api.v1P\x01\x62\x06proto3')



_ATOMICLITERAL = DESCRIPTOR.message_types_by_name['AtomicLiteral']
_FILTER = DESCRIPTOR.message_types_by_name['Filter']
_OR = DESCRIPTOR.message_types_by_name['Or']
_AND = DESCRIPTOR.message_types_by_name['And']
_EQUAL = DESCRIPTOR.message_types_by_name['Equal']
_NOTEQUAL = DESCRIPTOR.message_types_by_name['NotEqual']
_GREATERTHAN = DESCRIPTOR.message_types_by_name['GreaterThan']
_GREATERTHANEQUAL = DESCRIPTOR.message_types_by_name['GreaterThanEqual']
_LESSTHAN = DESCRIPTOR.message_types_by_name['LessThan']
_LESSTHANEQUAL = DESCRIPTOR.message_types_by_name['LessThanEqual']
_LIKE = DESCRIPTOR.message_types_by_name['Like']
_ISNULL = DESCRIPTOR.message_types_by_name['IsNull']
_ISNOTNULL = DESCRIPTOR.message_types_by_name['IsNotNull']
AtomicLiteral = _reflection.GeneratedProtocolMessageType('AtomicLiteral', (_message.Message,), {
  'DESCRIPTOR' : _ATOMICLITERAL,
  '__module__' : 'qwak.vectors.v1.filters_pb2'
  # @@protoc_insertion_point(class_scope:qwak.vector.store.serving.v1.AtomicLiteral)
  })
_sym_db.RegisterMessage(AtomicLiteral)

Filter = _reflection.GeneratedProtocolMessageType('Filter', (_message.Message,), {
  'DESCRIPTOR' : _FILTER,
  '__module__' : 'qwak.vectors.v1.filters_pb2'
  # @@protoc_insertion_point(class_scope:qwak.vector.store.serving.v1.Filter)
  })
_sym_db.RegisterMessage(Filter)

Or = _reflection.GeneratedProtocolMessageType('Or', (_message.Message,), {
  'DESCRIPTOR' : _OR,
  '__module__' : 'qwak.vectors.v1.filters_pb2'
  # @@protoc_insertion_point(class_scope:qwak.vector.store.serving.v1.Or)
  })
_sym_db.RegisterMessage(Or)

And = _reflection.GeneratedProtocolMessageType('And', (_message.Message,), {
  'DESCRIPTOR' : _AND,
  '__module__' : 'qwak.vectors.v1.filters_pb2'
  # @@protoc_insertion_point(class_scope:qwak.vector.store.serving.v1.And)
  })
_sym_db.RegisterMessage(And)

Equal = _reflection.GeneratedProtocolMessageType('Equal', (_message.Message,), {
  'DESCRIPTOR' : _EQUAL,
  '__module__' : 'qwak.vectors.v1.filters_pb2'
  # @@protoc_insertion_point(class_scope:qwak.vector.store.serving.v1.Equal)
  })
_sym_db.RegisterMessage(Equal)

NotEqual = _reflection.GeneratedProtocolMessageType('NotEqual', (_message.Message,), {
  'DESCRIPTOR' : _NOTEQUAL,
  '__module__' : 'qwak.vectors.v1.filters_pb2'
  # @@protoc_insertion_point(class_scope:qwak.vector.store.serving.v1.NotEqual)
  })
_sym_db.RegisterMessage(NotEqual)

GreaterThan = _reflection.GeneratedProtocolMessageType('GreaterThan', (_message.Message,), {
  'DESCRIPTOR' : _GREATERTHAN,
  '__module__' : 'qwak.vectors.v1.filters_pb2'
  # @@protoc_insertion_point(class_scope:qwak.vector.store.serving.v1.GreaterThan)
  })
_sym_db.RegisterMessage(GreaterThan)

GreaterThanEqual = _reflection.GeneratedProtocolMessageType('GreaterThanEqual', (_message.Message,), {
  'DESCRIPTOR' : _GREATERTHANEQUAL,
  '__module__' : 'qwak.vectors.v1.filters_pb2'
  # @@protoc_insertion_point(class_scope:qwak.vector.store.serving.v1.GreaterThanEqual)
  })
_sym_db.RegisterMessage(GreaterThanEqual)

LessThan = _reflection.GeneratedProtocolMessageType('LessThan', (_message.Message,), {
  'DESCRIPTOR' : _LESSTHAN,
  '__module__' : 'qwak.vectors.v1.filters_pb2'
  # @@protoc_insertion_point(class_scope:qwak.vector.store.serving.v1.LessThan)
  })
_sym_db.RegisterMessage(LessThan)

LessThanEqual = _reflection.GeneratedProtocolMessageType('LessThanEqual', (_message.Message,), {
  'DESCRIPTOR' : _LESSTHANEQUAL,
  '__module__' : 'qwak.vectors.v1.filters_pb2'
  # @@protoc_insertion_point(class_scope:qwak.vector.store.serving.v1.LessThanEqual)
  })
_sym_db.RegisterMessage(LessThanEqual)

Like = _reflection.GeneratedProtocolMessageType('Like', (_message.Message,), {
  'DESCRIPTOR' : _LIKE,
  '__module__' : 'qwak.vectors.v1.filters_pb2'
  # @@protoc_insertion_point(class_scope:qwak.vector.store.serving.v1.Like)
  })
_sym_db.RegisterMessage(Like)

IsNull = _reflection.GeneratedProtocolMessageType('IsNull', (_message.Message,), {
  'DESCRIPTOR' : _ISNULL,
  '__module__' : 'qwak.vectors.v1.filters_pb2'
  # @@protoc_insertion_point(class_scope:qwak.vector.store.serving.v1.IsNull)
  })
_sym_db.RegisterMessage(IsNull)

IsNotNull = _reflection.GeneratedProtocolMessageType('IsNotNull', (_message.Message,), {
  'DESCRIPTOR' : _ISNOTNULL,
  '__module__' : 'qwak.vectors.v1.filters_pb2'
  # @@protoc_insertion_point(class_scope:qwak.vector.store.serving.v1.IsNotNull)
  })
_sym_db.RegisterMessage(IsNotNull)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\032com.qwak.ai.vectors.api.v1P\001'
  _ATOMICLITERAL._serialized_start=97
  _ATOMICLITERAL._serialized_end=284
  _FILTER._serialized_start=287
  _FILTER._serialized_end=920
  _OR._serialized_start=922
  _OR._serialized_end=1031
  _AND._serialized_start=1033
  _AND._serialized_end=1143
  _EQUAL._serialized_start=1145
  _EQUAL._serialized_end=1230
  _NOTEQUAL._serialized_start=1232
  _NOTEQUAL._serialized_end=1320
  _GREATERTHAN._serialized_start=1322
  _GREATERTHAN._serialized_end=1413
  _GREATERTHANEQUAL._serialized_start=1415
  _GREATERTHANEQUAL._serialized_end=1511
  _LESSTHAN._serialized_start=1513
  _LESSTHAN._serialized_end=1601
  _LESSTHANEQUAL._serialized_start=1603
  _LESSTHANEQUAL._serialized_end=1696
  _LIKE._serialized_start=1698
  _LIKE._serialized_end=1739
  _ISNULL._serialized_start=1741
  _ISNULL._serialized_end=1767
  _ISNOTNULL._serialized_start=1769
  _ISNOTNULL._serialized_end=1798
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class ServiceLocationDescriptor(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SERVICE_URL_FIELD_NUMBER: builtins.int
    LOCATION_FIELD_NUMBER: builtins.int
    service_url: builtins.str
    @property
    def location(self) -> global___ServiceLocation: ...
    def __init__(
        self,
        *,
        service_url: builtins.str = ...,
        location: global___ServiceLocation | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["location", b"location"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["location", b"location", "service_url", b"service_url"]) -> None: ...

global___ServiceLocationDescriptor = ServiceLocationDescriptor

class ServiceLocation(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CORE_FIELD_NUMBER: builtins.int
    EDGE_FIELD_NUMBER: builtins.int
    @property
    def core(self) -> global___Core: ...
    @property
    def edge(self) -> global___Edge: ...
    def __init__(
        self,
        *,
        core: global___Core | None = ...,
        edge: global___Edge | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["core", b"core", "edge", b"edge", "service_location", b"service_location"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["core", b"core", "edge", b"edge", "service_location", b"service_location"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["service_location", b"service_location"]) -> typing_extensions.Literal["core", "edge"] | None: ...

global___ServiceLocation = ServiceLocation

class Core(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___Core = Core

class Edge(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___Edge = Edge



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/administration/account/v1/jfrog_tenant_details.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n9qwak/administration/account/v1/jfrog_tenant_details.proto\x12\x1eqwak.administration.account.v1\"\xa6\x02\n\x12JfrogTenantDetails\x12\x11\n\ttenant_id\x18\x01 \x01(\t\x12^\n\x1d\x64\x65\x66\x61ult_jfrog_project_details\x18\x02 \x01(\x0b\x32\x33.qwak.administration.account.v1.JfrogProjectDetailsB\x02\x18\x01\x12\\\n\x1ajfrog_tenant_configuration\x18\x03 \x01(\x0b\x32\x38.qwak.administration.account.v1.JFrogTenantConfiguration\x12?\n\x0bjpd_details\x18\x04 \x01(\x0b\x32*.qwak.administration.account.v1.JPDDetails\"\xb0\x01\n\nJPDDetails\x12N\n\x0esh_jpd_details\x18\x01 \x01(\x0b\x32\x34.qwak.administration.account.v1.SelfHostedJPDDetailsH\x00\x12J\n\x10saas_jpd_details\x18\x02 \x01(\x0b\x32..qwak.administration.account.v1.SAASJPDDetailsH\x00\x42\x06\n\x04type\"(\n\x14SelfHostedJPDDetails\x12\x10\n\x08\x62\x61se_url\x18\x01 \x01(\t\"\x10\n\x0eSAASJPDDetails\"v\n\x18JFrogTenantConfiguration\x12Z\n\x1d\x64\x65\x66\x61ult_jfrog_project_details\x18\x01 \x01(\x0b\x32\x33.qwak.administration.account.v1.JfrogProjectDetails\"*\n\x13JfrogProjectDetails\x12\x13\n\x0bproject_key\x18\x01 \x01(\t\"\xbe\x01\n\x0fJfrogTenantSpec\x12\x11\n\ttenant_id\x18\x01 \x01(\t\x12N\n\x14\x64\x65\x66\x61ult_project_spec\x18\x02 \x01(\x0b\x32\x30.qwak.administration.account.v1.JfrogProjectSpec\x12H\n\x10jpd_details_spec\x18\x03 \x01(\x0b\x32..qwak.administration.account.v1.JPDDetailsSpec\"\xc6\x01\n\x0eJPDDetailsSpec\x12W\n\x13sh_jpd_details_spec\x18\x01 \x01(\x0b\x32\x38.qwak.administration.account.v1.SelfHostedJPDDetailsSpecH\x00\x12S\n\x15saas_jpd_details_spec\x18\x02 \x01(\x0b\x32\x32.qwak.administration.account.v1.SAASJPDDetailsSpecH\x00\x42\x06\n\x04type\",\n\x18SelfHostedJPDDetailsSpec\x12\x10\n\x08\x62\x61se_url\x18\x01 \x01(\t\"\x14\n\x12SAASJPDDetailsSpec\"\'\n\x10JfrogProjectSpec\x12\x13\n\x0bproject_key\x18\x01 \x01(\tB\xb4\x01\n)com.qwak.ai.administration.api.account.v1P\x01Z\x84\x01github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/administration/account/v1;account_v0b\x06proto3')



_JFROGTENANTDETAILS = DESCRIPTOR.message_types_by_name['JfrogTenantDetails']
_JPDDETAILS = DESCRIPTOR.message_types_by_name['JPDDetails']
_SELFHOSTEDJPDDETAILS = DESCRIPTOR.message_types_by_name['SelfHostedJPDDetails']
_SAASJPDDETAILS = DESCRIPTOR.message_types_by_name['SAASJPDDetails']
_JFROGTENANTCONFIGURATION = DESCRIPTOR.message_types_by_name['JFrogTenantConfiguration']
_JFROGPROJECTDETAILS = DESCRIPTOR.message_types_by_name['JfrogProjectDetails']
_JFROGTENANTSPEC = DESCRIPTOR.message_types_by_name['JfrogTenantSpec']
_JPDDETAILSSPEC = DESCRIPTOR.message_types_by_name['JPDDetailsSpec']
_SELFHOSTEDJPDDETAILSSPEC = DESCRIPTOR.message_types_by_name['SelfHostedJPDDetailsSpec']
_SAASJPDDETAILSSPEC = DESCRIPTOR.message_types_by_name['SAASJPDDetailsSpec']
_JFROGPROJECTSPEC = DESCRIPTOR.message_types_by_name['JfrogProjectSpec']
JfrogTenantDetails = _reflection.GeneratedProtocolMessageType('JfrogTenantDetails', (_message.Message,), {
  'DESCRIPTOR' : _JFROGTENANTDETAILS,
  '__module__' : 'qwak.administration.account.v1.jfrog_tenant_details_pb2'
  # @@protoc_insertion_point(class_scope:qwak.administration.account.v1.JfrogTenantDetails)
  })
_sym_db.RegisterMessage(JfrogTenantDetails)

JPDDetails = _reflection.GeneratedProtocolMessageType('JPDDetails', (_message.Message,), {
  'DESCRIPTOR' : _JPDDETAILS,
  '__module__' : 'qwak.administration.account.v1.jfrog_tenant_details_pb2'
  # @@protoc_insertion_point(class_scope:qwak.administration.account.v1.JPDDetails)
  })
_sym_db.RegisterMessage(JPDDetails)

SelfHostedJPDDetails = _reflection.GeneratedProtocolMessageType('SelfHostedJPDDetails', (_message.Message,), {
  'DESCRIPTOR' : _SELFHOSTEDJPDDETAILS,
  '__module__' : 'qwak.administration.account.v1.jfrog_tenant_details_pb2'
  # @@protoc_insertion_point(class_scope:qwak.administration.account.v1.SelfHostedJPDDetails)
  })
_sym_db.RegisterMessage(SelfHostedJPDDetails)

SAASJPDDetails = _reflection.GeneratedProtocolMessageType('SAASJPDDetails', (_message.Message,), {
  'DESCRIPTOR' : _SAASJPDDETAILS,
  '__module__' : 'qwak.administration.account.v1.jfrog_tenant_details_pb2'
  # @@protoc_insertion_point(class_scope:qwak.administration.account.v1.SAASJPDDetails)
  })
_sym_db.RegisterMessage(SAASJPDDetails)

JFrogTenantConfiguration = _reflection.GeneratedProtocolMessageType('JFrogTenantConfiguration', (_message.Message,), {
  'DESCRIPTOR' : _JFROGTENANTCONFIGURATION,
  '__module__' : 'qwak.administration.account.v1.jfrog_tenant_details_pb2'
  # @@protoc_insertion_point(class_scope:qwak.administration.account.v1.JFrogTenantConfiguration)
  })
_sym_db.RegisterMessage(JFrogTenantConfiguration)

JfrogProjectDetails = _reflection.GeneratedProtocolMessageType('JfrogProjectDetails', (_message.Message,), {
  'DESCRIPTOR' : _JFROGPROJECTDETAILS,
  '__module__' : 'qwak.administration.account.v1.jfrog_tenant_details_pb2'
  # @@protoc_insertion_point(class_scope:qwak.administration.account.v1.JfrogProjectDetails)
  })
_sym_db.RegisterMessage(JfrogProjectDetails)

JfrogTenantSpec = _reflection.GeneratedProtocolMessageType('JfrogTenantSpec', (_message.Message,), {
  'DESCRIPTOR' : _JFROGTENANTSPEC,
  '__module__' : 'qwak.administration.account.v1.jfrog_tenant_details_pb2'
  # @@protoc_insertion_point(class_scope:qwak.administration.account.v1.JfrogTenantSpec)
  })
_sym_db.RegisterMessage(JfrogTenantSpec)

JPDDetailsSpec = _reflection.GeneratedProtocolMessageType('JPDDetailsSpec', (_message.Message,), {
  'DESCRIPTOR' : _JPDDETAILSSPEC,
  '__module__' : 'qwak.administration.account.v1.jfrog_tenant_details_pb2'
  # @@protoc_insertion_point(class_scope:qwak.administration.account.v1.JPDDetailsSpec)
  })
_sym_db.RegisterMessage(JPDDetailsSpec)

SelfHostedJPDDetailsSpec = _reflection.GeneratedProtocolMessageType('SelfHostedJPDDetailsSpec', (_message.Message,), {
  'DESCRIPTOR' : _SELFHOSTEDJPDDETAILSSPEC,
  '__module__' : 'qwak.administration.account.v1.jfrog_tenant_details_pb2'
  # @@protoc_insertion_point(class_scope:qwak.administration.account.v1.SelfHostedJPDDetailsSpec)
  })
_sym_db.RegisterMessage(SelfHostedJPDDetailsSpec)

SAASJPDDetailsSpec = _reflection.GeneratedProtocolMessageType('SAASJPDDetailsSpec', (_message.Message,), {
  'DESCRIPTOR' : _SAASJPDDETAILSSPEC,
  '__module__' : 'qwak.administration.account.v1.jfrog_tenant_details_pb2'
  # @@protoc_insertion_point(class_scope:qwak.administration.account.v1.SAASJPDDetailsSpec)
  })
_sym_db.RegisterMessage(SAASJPDDetailsSpec)

JfrogProjectSpec = _reflection.GeneratedProtocolMessageType('JfrogProjectSpec', (_message.Message,), {
  'DESCRIPTOR' : _JFROGPROJECTSPEC,
  '__module__' : 'qwak.administration.account.v1.jfrog_tenant_details_pb2'
  # @@protoc_insertion_point(class_scope:qwak.administration.account.v1.JfrogProjectSpec)
  })
_sym_db.RegisterMessage(JfrogProjectSpec)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n)com.qwak.ai.administration.api.account.v1P\001Z\204\001github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/administration/account/v1;account_v0'
  _JFROGTENANTDETAILS.fields_by_name['default_jfrog_project_details']._options = None
  _JFROGTENANTDETAILS.fields_by_name['default_jfrog_project_details']._serialized_options = b'\030\001'
  _JFROGTENANTDETAILS._serialized_start=94
  _JFROGTENANTDETAILS._serialized_end=388
  _JPDDETAILS._serialized_start=391
  _JPDDETAILS._serialized_end=567
  _SELFHOSTEDJPDDETAILS._serialized_start=569
  _SELFHOSTEDJPDDETAILS._serialized_end=609
  _SAASJPDDETAILS._serialized_start=611
  _SAASJPDDETAILS._serialized_end=627
  _JFROGTENANTCONFIGURATION._serialized_start=629
  _JFROGTENANTCONFIGURATION._serialized_end=747
  _JFROGPROJECTDETAILS._serialized_start=749
  _JFROGPROJECTDETAILS._serialized_end=791
  _JFROGTENANTSPEC._serialized_start=794
  _JFROGTENANTSPEC._serialized_end=984
  _JPDDETAILSSPEC._serialized_start=987
  _JPDDETAILSSPEC._serialized_end=1185
  _SELFHOSTEDJPDDETAILSSPEC._serialized_start=1187
  _SELFHOSTEDJPDDETAILSSPEC._serialized_end=1231
  _SAASJPDDETAILSSPEC._serialized_start=1233
  _SAASJPDDETAILSSPEC._serialized_end=1253
  _JFROGPROJECTSPEC._serialized_start=1255
  _JFROGPROJECTSPEC._serialized_end=1294
# @@protoc_insertion_point(module_scope)



#
#    Copyright 2024 OpenAI
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from dataclasses import dataclass
from typing import Dict, List, Optional

from typing_extensions import Literal

__all__ = ["CompletionChoice", "Logprobs"]


@dataclass
class Logprobs:
    text_offset: Optional[List[int]] = None

    token_logprobs: Optional[List[float]] = None

    tokens: Optional[List[str]] = None

    top_logprobs: Optional[List[Dict[str, float]]] = None


@dataclass
class CompletionChoice:
    finish_reason: Literal["stop", "length", "content_filter"]
    """The reason the model stopped generating tokens.

    This will be `stop` if the model hit a natural stop point or a provided stop
    sequence, `length` if the maximum number of tokens specified in the request was
    reached, or `content_filter` if content was omitted due to a flag from our
    content filters.
    """
    index: int
    text: str
    logprobs: Optional[Logprobs] = None



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/administration/runtime_configuration/v0/creds/secret.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n?qwak/administration/runtime_configuration/v0/creds/secret.proto\x12\x39qwak.administration.runtime_configuration.v0.creds.secret\"\x8e\x01\n\x0cSecretKeyRef\x12v\n\x19kubernetes_secret_key_ref\x18\x01 \x01(\x0b\x32Q.qwak.administration.runtime_configuration.v0.creds.secret.KubernetesSecretKeyRefH\x00\x42\x06\n\x04type\"3\n\x16KubernetesSecretKeyRef\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\tB\xe5\x01\n2com.jfrog.ml.runtime_configuration.v0.creds.secretP\x01Z\xac\x01github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/administration/runtime_configuration/v0/creds;runtime_configuration_v0_credsb\x06proto3')



_SECRETKEYREF = DESCRIPTOR.message_types_by_name['SecretKeyRef']
_KUBERNETESSECRETKEYREF = DESCRIPTOR.message_types_by_name['KubernetesSecretKeyRef']
SecretKeyRef = _reflection.GeneratedProtocolMessageType('SecretKeyRef', (_message.Message,), {
  'DESCRIPTOR' : _SECRETKEYREF,
  '__module__' : 'qwak.administration.runtime_configuration.v0.creds.secret_pb2'
  # @@protoc_insertion_point(class_scope:qwak.administration.runtime_configuration.v0.creds.secret.SecretKeyRef)
  })
_sym_db.RegisterMessage(SecretKeyRef)

KubernetesSecretKeyRef = _reflection.GeneratedProtocolMessageType('KubernetesSecretKeyRef', (_message.Message,), {
  'DESCRIPTOR' : _KUBERNETESSECRETKEYREF,
  '__module__' : 'qwak.administration.runtime_configuration.v0.creds.secret_pb2'
  # @@protoc_insertion_point(class_scope:qwak.administration.runtime_configuration.v0.creds.secret.KubernetesSecretKeyRef)
  })
_sym_db.RegisterMessage(KubernetesSecretKeyRef)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n2com.jfrog.ml.runtime_configuration.v0.creds.secretP\001Z\254\001github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/administration/runtime_configuration/v0/creds;runtime_configuration_v0_creds'
  _SECRETKEYREF._serialized_start=127
  _SECRETKEYREF._serialized_end=269
  _KUBERNETESSECRETKEYREF._serialized_start=271
  _KUBERNETESSECRETKEYREF._serialized_end=322
# @@protoc_insertion_point(module_scope)



from google.protobuf.message import Message

from .base_output_adapter import BaseOutputAdapter


class ProtoOutputAdapter(BaseOutputAdapter):
    @staticmethod
    def http_content_type():
        return {"Content-Type": "application/octet-stream"}

    @staticmethod
    def pack_user_func_return_value(return_result: Message) -> bytes:
        try:
            return return_result.SerializeToString()
        except Exception as e:
            raise ValueError(f"Failed to serialize proto message. Error is {e}")



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class BatchIngestion(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURESET_NAME_FIELD_NUMBER: builtins.int
    MANUAL_TRIGGER_FIELD_NUMBER: builtins.int
    SCHEDULED_TRIGGER_FIELD_NUMBER: builtins.int
    featureset_name: builtins.str
    @property
    def manual_trigger(self) -> global___ManualTrigger:
        """Manual Trigger i.e., RunNow"""
    @property
    def scheduled_trigger(self) -> global___ScheduledTrigger:
        """Scheduled Trigger (e.g. Airflow)."""
    def __init__(
        self,
        *,
        featureset_name: builtins.str = ...,
        manual_trigger: global___ManualTrigger | None = ...,
        scheduled_trigger: global___ScheduledTrigger | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["manual_trigger", b"manual_trigger", "scheduled_trigger", b"scheduled_trigger", "trigger_type", b"trigger_type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["featureset_name", b"featureset_name", "manual_trigger", b"manual_trigger", "scheduled_trigger", b"scheduled_trigger", "trigger_type", b"trigger_type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["trigger_type", b"trigger_type"]) -> typing_extensions.Literal["manual_trigger", "scheduled_trigger"] | None: ...

global___BatchIngestion = BatchIngestion

class ManualTrigger(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TRIGGER_DATE_FIELD_NUMBER: builtins.int
    @property
    def trigger_date(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """timestamp at which the manual run was triggered. set by the service (ignored if set by the user)."""
    def __init__(
        self,
        *,
        trigger_date: google.protobuf.timestamp_pb2.Timestamp | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["trigger_date", b"trigger_date"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["trigger_date", b"trigger_date"]) -> None: ...

global___ManualTrigger = ManualTrigger

class ScheduledTrigger(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LOGICAL_BATCH_DATE_FIELD_NUMBER: builtins.int
    @property
    def logical_batch_date(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """logical timestamp of the batch"""
    def __init__(
        self,
        *,
        logical_batch_date: google.protobuf.timestamp_pb2.Timestamp | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["logical_batch_date", b"logical_batch_date"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["logical_batch_date", b"logical_batch_date"]) -> None: ...

global___ScheduledTrigger = ScheduledTrigger



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class JfrogTenantDetails(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TENANT_ID_FIELD_NUMBER: builtins.int
    DEFAULT_JFROG_PROJECT_DETAILS_FIELD_NUMBER: builtins.int
    JFROG_TENANT_CONFIGURATION_FIELD_NUMBER: builtins.int
    JPD_DETAILS_FIELD_NUMBER: builtins.int
    tenant_id: builtins.str
    """The JFrog tenant's ID"""
    @property
    def default_jfrog_project_details(self) -> global___JfrogProjectDetails:
        """JFrog project details"""
    @property
    def jfrog_tenant_configuration(self) -> global___JFrogTenantConfiguration:
        """The JFrog tenant's configuration"""
    @property
    def jpd_details(self) -> global___JPDDetails:
        """Details about this tenant id JPD"""
    def __init__(
        self,
        *,
        tenant_id: builtins.str = ...,
        default_jfrog_project_details: global___JfrogProjectDetails | None = ...,
        jfrog_tenant_configuration: global___JFrogTenantConfiguration | None = ...,
        jpd_details: global___JPDDetails | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["default_jfrog_project_details", b"default_jfrog_project_details", "jfrog_tenant_configuration", b"jfrog_tenant_configuration", "jpd_details", b"jpd_details"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["default_jfrog_project_details", b"default_jfrog_project_details", "jfrog_tenant_configuration", b"jfrog_tenant_configuration", "jpd_details", b"jpd_details", "tenant_id", b"tenant_id"]) -> None: ...

global___JfrogTenantDetails = JfrogTenantDetails

class JPDDetails(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SH_JPD_DETAILS_FIELD_NUMBER: builtins.int
    SAAS_JPD_DETAILS_FIELD_NUMBER: builtins.int
    @property
    def sh_jpd_details(self) -> global___SelfHostedJPDDetails: ...
    @property
    def saas_jpd_details(self) -> global___SAASJPDDetails: ...
    def __init__(
        self,
        *,
        sh_jpd_details: global___SelfHostedJPDDetails | None = ...,
        saas_jpd_details: global___SAASJPDDetails | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["saas_jpd_details", b"saas_jpd_details", "sh_jpd_details", b"sh_jpd_details", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["saas_jpd_details", b"saas_jpd_details", "sh_jpd_details", b"sh_jpd_details", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["type", b"type"]) -> typing_extensions.Literal["sh_jpd_details", "saas_jpd_details"] | None: ...

global___JPDDetails = JPDDetails

class SelfHostedJPDDetails(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BASE_URL_FIELD_NUMBER: builtins.int
    base_url: builtins.str
    """The JPD base url, reachable from data planes"""
    def __init__(
        self,
        *,
        base_url: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["base_url", b"base_url"]) -> None: ...

global___SelfHostedJPDDetails = SelfHostedJPDDetails

class SAASJPDDetails(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___SAASJPDDetails = SAASJPDDetails

class JFrogTenantConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DEFAULT_JFROG_PROJECT_DETAILS_FIELD_NUMBER: builtins.int
    @property
    def default_jfrog_project_details(self) -> global___JfrogProjectDetails:
        """JFrog project details"""
    def __init__(
        self,
        *,
        default_jfrog_project_details: global___JfrogProjectDetails | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["default_jfrog_project_details", b"default_jfrog_project_details"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["default_jfrog_project_details", b"default_jfrog_project_details"]) -> None: ...

global___JFrogTenantConfiguration = JFrogTenantConfiguration

class JfrogProjectDetails(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_KEY_FIELD_NUMBER: builtins.int
    project_key: builtins.str
    """The JFrog project's key"""
    def __init__(
        self,
        *,
        project_key: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["project_key", b"project_key"]) -> None: ...

global___JfrogProjectDetails = JfrogProjectDetails

class JfrogTenantSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TENANT_ID_FIELD_NUMBER: builtins.int
    DEFAULT_PROJECT_SPEC_FIELD_NUMBER: builtins.int
    JPD_DETAILS_SPEC_FIELD_NUMBER: builtins.int
    tenant_id: builtins.str
    """The JFrog tenant's ID"""
    @property
    def default_project_spec(self) -> global___JfrogProjectSpec:
        """The JFrog tenant's default project details"""
    @property
    def jpd_details_spec(self) -> global___JPDDetailsSpec:
        """Details about this tenant id JPD"""
    def __init__(
        self,
        *,
        tenant_id: builtins.str = ...,
        default_project_spec: global___JfrogProjectSpec | None = ...,
        jpd_details_spec: global___JPDDetailsSpec | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["default_project_spec", b"default_project_spec", "jpd_details_spec", b"jpd_details_spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["default_project_spec", b"default_project_spec", "jpd_details_spec", b"jpd_details_spec", "tenant_id", b"tenant_id"]) -> None: ...

global___JfrogTenantSpec = JfrogTenantSpec

class JPDDetailsSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SH_JPD_DETAILS_SPEC_FIELD_NUMBER: builtins.int
    SAAS_JPD_DETAILS_SPEC_FIELD_NUMBER: builtins.int
    @property
    def sh_jpd_details_spec(self) -> global___SelfHostedJPDDetailsSpec: ...
    @property
    def saas_jpd_details_spec(self) -> global___SAASJPDDetailsSpec: ...
    def __init__(
        self,
        *,
        sh_jpd_details_spec: global___SelfHostedJPDDetailsSpec | None = ...,
        saas_jpd_details_spec: global___SAASJPDDetailsSpec | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["saas_jpd_details_spec", b"saas_jpd_details_spec", "sh_jpd_details_spec", b"sh_jpd_details_spec", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["saas_jpd_details_spec", b"saas_jpd_details_spec", "sh_jpd_details_spec", b"sh_jpd_details_spec", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["type", b"type"]) -> typing_extensions.Literal["sh_jpd_details_spec", "saas_jpd_details_spec"] | None: ...

global___JPDDetailsSpec = JPDDetailsSpec

class SelfHostedJPDDetailsSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BASE_URL_FIELD_NUMBER: builtins.int
    base_url: builtins.str
    """The JPD base url, reachable from data planes"""
    def __init__(
        self,
        *,
        base_url: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["base_url", b"base_url"]) -> None: ...

global___SelfHostedJPDDetailsSpec = SelfHostedJPDDetailsSpec

class SAASJPDDetailsSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___SAASJPDDetailsSpec = SAASJPDDetailsSpec

class JfrogProjectSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_KEY_FIELD_NUMBER: builtins.int
    project_key: builtins.str
    """The JFrog project's key"""
    def __init__(
        self,
        *,
        project_key: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["project_key", b"project_key"]) -> None: ...

global___JfrogProjectSpec = JfrogProjectSpec



import uuid
from typing import List, Optional

from _qwak_proto.qwak.automation.v1.automation_management_service_pb2 import (
    CreateAutomationRequest,
    DeleteAutomationRequest,
    GetAutomationByNameRequest,
    GetAutomationRequest,
    ListAutomationExecutionsRequest,
    ListAutomationsRequest,
    RegisterAutomationExecutionRequest,
    RunAutomationRequest,
    UpdateAutomationExecutionRequest,
    UpdateAutomationRequest,
)
from _qwak_proto.qwak.automation.v1.automation_management_service_pb2_grpc import (
    AutomationManagementServiceStub,
)
from _qwak_proto.qwak.automation.v1.automation_pb2 import Automation as AutomationProto
from dependency_injector.wiring import Provide
from grpc import RpcError, StatusCode
from qwak.automations.automation_executions import (
    AutomationExecution,
    ExecutionRunDetails,
)
from qwak.automations.automations import Automation
from qwak.exceptions import QwakException
from qwak.inner.di_configuration import QwakContainer


class AutomationsManagementClient:
    """
    Used for interacting with Automations Registry endpoints
    """

    def __init__(self, grpc_channel=Provide[QwakContainer.core_grpc_channel]):
        self._service_stub = AutomationManagementServiceStub(grpc_channel)

    def get_automation_by_name(self, automation_name: str) -> Optional[Automation]:
        """

        Args:
            automation_name: automation name to get
        Returns:
            Automation
        """
        try:
            automation: AutomationProto = self._service_stub.GetAutomationByName(
                GetAutomationByNameRequest(automation_name=automation_name)
            ).automation
            return Automation.from_proto(automation)
        except RpcError as e:
            if e.code() == StatusCode.NOT_FOUND:
                return None
            else:
                raise QwakException(
                    f"Failed to fetch automation by name [{automation_name}], error code: [{e.details()}]"
                )

    def get_automation_by_id(self, automation_id: str) -> Optional[Automation]:
        """

        Args:
            automation_id: automation name to get
        Returns:
            Automation
        """
        try:
            automation: AutomationProto = self._service_stub.GetAutomation(
                GetAutomationRequest(automation_id=automation_id)
            ).automation
            return Automation.from_proto(automation)
        except RpcError as e:
            if e.code() == StatusCode.NOT_FOUND:
                return None
            else:
                raise QwakException(
                    f"Failed to fetch automation with ID [{automation_id}], error code: [{e.details()}]"
                )

    def update_automation(
        self, automation_id: str, automation: AutomationProto
    ) -> None:
        """

        Args:
            automation: automation to update
        """
        automation.automation_spec.is_sdk_v1 = True
        try:
            self._service_stub.UpdateAutomation(
                UpdateAutomationRequest(
                    automation_id=automation_id,
                    automation_spec=automation.automation_spec,
                )
            )
            return Automation.from_proto(automation)
        except RpcError as e:
            raise QwakException(
                f"Failed to update automation with ID [{automation.automation_id}], error code: [{e.details()}]"
            )

    def create_automation(self, automation: AutomationProto) -> str:
        """

        Args:
            automation: automation to create
        Returns:
            Automation id of created automation
        """
        automation.automation_spec.is_sdk_v1 = True
        try:
            return self._service_stub.CreateAutomation(
                CreateAutomationRequest(automation_spec=automation.automation_spec)
            ).automation_id
        except RpcError as e:
            raise QwakException(
                f"Failed to update automation, error code: [{e.details()}]"
            )

    def run_automation(self, automation_id: str) -> str:
        """
        Trigger an automation to run

        Args:
            automation_id: automation id to trigger
        """
        try:
            return self._service_stub.RunAutomation(
                RunAutomationRequest(automation_id=automation_id)
            )
        except RpcError as e:
            raise QwakException(
                f"Failed to execute a run automation command, error code: [{e.details()}]"
            )

    def list_automations(
        self,
    ) -> Optional[List[Automation]]:
        """

        Returns:
            List of client automations
        """
        try:
            automations = self.get_list_automations_from_server()
            return list(
                map(lambda automation: Automation.from_proto(automation), automations)
            )
        except RpcError as e:
            if e.code() == StatusCode.NOT_FOUND:
                return None
            else:
                raise QwakException(
                    f"Failed to fetch automations, error code: [{e.details()}]"
                )

    def get_list_automations_from_server(self):
        """
        Get list of automations from server
        """
        automations: List[AutomationProto] = self._service_stub.ListAutomations(
            ListAutomationsRequest()
        ).automations
        return automations

    def delete_automation(self, automation_id: str) -> bool:
        """

        Args:
            automation_id: The automation id to delete

        Returns:
            boolean whether the automation was deleted or not
        """
        try:
            self._service_stub.DeleteAutomation(
                DeleteAutomationRequest(automation_id=automation_id)
            )
            return True
        except RpcError as e:
            if e.code() == StatusCode.NOT_FOUND:
                return False
            else:
                raise QwakException(
                    f"Failed to fetch automations, error code: [{e.details()}]"
                )

    def register_execution(self, automation_id: str) -> str:
        """

        Args:
            automation_id: The automation id to register

        Returns:
            The execution id for the automation execution

        """
        try:
            execution_id = str(uuid.uuid4())
            self._service_stub.RegisterAutomationExecution(
                RegisterAutomationExecutionRequest(
                    automation_id=automation_id, execution_id=execution_id
                )
            )
            return execution_id
        except RpcError as e:
            if e.code() == StatusCode.NOT_FOUND:
                raise QwakException(
                    f"Failed to register automation execution, automation {automation_id} was not found"
                )
            else:
                raise QwakException(
                    f"Failed to register automation execution, error code: [{e.details()}]"
                )

    def update_execution(
        self, execution_id: str, run_details: ExecutionRunDetails
    ) -> None:
        """

        Args:
            execution_id: The execution id to update
            run_details: The details to update on the execution

        """
        try:
            self._service_stub.UpdateAutomationExecution(
                UpdateAutomationExecutionRequest(
                    execution_id=execution_id, run_details=run_details.to_proto()
                )
            )
        except RpcError as e:
            if e.code() == StatusCode.NOT_FOUND:
                raise QwakException(
                    f"Failed to update execution, execution {execution_id} was not found"
                )
            else:
                raise QwakException(
                    f"Failed to update execution, error code: [{e.details()}]"
                )

    def list_executions(self, automation_id: str) -> List[AutomationExecution]:
        """

        Args:
            automation_id: The automation to list its executions

        Returns:
            A list of executions for the specified automation

        """
        try:
            executions = self._service_stub.ListAutomationExecutions(
                ListAutomationExecutionsRequest(automation_id=automation_id)
            ).automation_executions
            return list(
                map(
                    lambda execution: AutomationExecution.from_proto(execution),
                    executions,
                )
            )
        except RpcError as e:
            if e.code() == StatusCode.NOT_FOUND:
                raise QwakException(
                    f"Failed to list executions for automation, automation {automation_id} was not found"
                )
            else:
                raise QwakException(
                    f"Failed to list executions for automation, error code: [{e.details()}]"
                )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/feature_store/features/feature_set_attribute.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.feature_store.v1.common.source_code import source_code_pb2 as qwak_dot_feature__store_dot_v1_dot_common_dot_source__code_dot_source__code__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n7qwak/feature_store/features/feature_set_attribute.proto\x12\x1bqwak.feature.store.features\x1a:qwak/feature_store/v1/common/source_code/source_code.proto\"j\n\x14\x46\x65\x61turesetAttributes\x12R\n\x10source_code_spec\x18\x01 \x01(\x0b\x32\x38.qwak.feature.store.v1.common.source_code.SourceCodeSpecB[\n&com.qwak.ai.feature.store.features.apiP\x01Z/qwak/featurestore/features;featurestorefeaturesb\x06proto3')



_FEATURESETATTRIBUTES = DESCRIPTOR.message_types_by_name['FeaturesetAttributes']
FeaturesetAttributes = _reflection.GeneratedProtocolMessageType('FeaturesetAttributes', (_message.Message,), {
  'DESCRIPTOR' : _FEATURESETATTRIBUTES,
  '__module__' : 'qwak.feature_store.features.feature_set_attribute_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.FeaturesetAttributes)
  })
_sym_db.RegisterMessage(FeaturesetAttributes)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n&com.qwak.ai.feature.store.features.apiP\001Z/qwak/featurestore/features;featurestorefeatures'
  _FEATURESETATTRIBUTES._serialized_start=148
  _FEATURESETATTRIBUTES._serialized_end=254
# @@protoc_insertion_point(module_scope)



from typing import Any, Type

from .base_output import BaseOutputAdapter


def detect_suitable_adapter(result) -> Type[BaseOutputAdapter]:
    try:
        import pandas as pd

        if isinstance(result, (pd.DataFrame, pd.Series)):
            from .dataframe_output import DataFrameOutput

            return DataFrameOutput
    except ImportError:
        pass

    try:
        import tensorflow as tf

        if isinstance(result, tf.Tensor):
            from .tensorflow_tensor_output import TfTensorOutput

            return TfTensorOutput
    except ImportError:
        pass

    from .json_output import JsonOutput

    return JsonOutput


class DefaultOutput(BaseOutputAdapter):
    """
    Detect suitable output adapter automatically
    OutputAdapter converts returns of user defined API function into specific output,
    such as HTTP response, command line stdout or AWS Lambda event object.

    Args:
        cors (str): DEPRECATED. Moved to the configuration file.
            The value of the Access-Control-Allow-Origin header set in the
            HTTP/AWS Lambda response object. If set to None, the header will not be set.
            Default is None.
    """

    def __init__(self, **kwargs):
        super(DefaultOutput, self).__init__(**kwargs)
        self.actual_adapter = None
        from .json_output import JsonOutput

        self.backup_adapter = JsonOutput()

    def pack_user_func_return_value(self, return_result: Any) -> Any:
        if self.actual_adapter is None:
            self.actual_adapter = detect_suitable_adapter(return_result)()
        if self.actual_adapter:
            return self.actual_adapter.pack_user_func_return_value(return_result)

        raise NotImplementedError()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/kube_deployment_captain/feature_set_deployment.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.feature_store.features import feature_set_types_pb2 as qwak_dot_feature__store_dot_features_dot_feature__set__types__pb2
from _qwak_proto.qwak.feature_store.entities import entity_pb2 as qwak_dot_feature__store_dot_entities_dot_entity__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n9qwak/kube_deployment_captain/feature_set_deployment.proto\x12\x1cqwak.kube.deployment.captain\x1a\x33qwak/feature_store/features/feature_set_types.proto\x1a(qwak/feature_store/entities/entity.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\x81\x07\n\x14\x46\x65\x61tureSetDeployment\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\x01 \x01(\t\x12S\n\x15streaming_feature_set\x18\x02 \x01(\x0b\x32\x32.qwak.feature.store.features.StreamingFeatureSetV1H\x00\x12K\n\x11\x62\x61tch_feature_set\x18\x03 \x01(\x0b\x32..qwak.feature.store.features.BatchFeatureSetV1H\x00\x12h\n!streaming_aggregation_feature_set\x18\x04 \x01(\x0b\x32;.qwak.feature.store.features.StreamingAggregationFeatureSetH\x00\x12|\n\x1e\x65xtra_deployment_configuration\x18\x05 \x03(\x0b\x32T.qwak.kube.deployment.captain.FeatureSetDeployment.ExtraDeploymentConfigurationEntry\x12n\n\x17\x65xtra_env_configuration\x18\x06 \x03(\x0b\x32M.qwak.kube.deployment.captain.FeatureSetDeployment.ExtraEnvConfigurationEntry\x12=\n\x06\x65ntity\x18\x07 \x01(\x0b\x32-.qwak.feature.store.entities.EntityDefinition\x12\x15\n\rfeatureset_id\x18\x08 \x01(\t\x12\x1a\n\x12secret_service_url\x18\t \x01(\t\x12\x0e\n\x06run_id\x18\n \x01(\x03\x12&\n\x1eqwak_internal_protocol_version\x18\x0b \x01(\x05\x12\x1a\n\x12\x66\x65\x61tureset_version\x18\x0c \x01(\x05\x1a\x43\n!ExtraDeploymentConfigurationEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a<\n\x1a\x45xtraEnvConfigurationEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\r\n\x0b\x66\x65\x61ture_set\"\x94\x01\n\x0c\x42\x61tchDetails\x12\x38\n\x14\x62\x61tch_execution_date\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x12\n\ntry_number\x18\x02 \x01(\x05\x12\x36\n\x12\x62\x61tch_trigger_date\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"+\n\x13\x45xecutionIdentifier\x12\x14\n\x0c\x65xecution_id\x18\x01 \x01(\t\"\x84\x02\n\x18\x46\x65\x61tureSetDeploymentSpec\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\t\x12\x46\n\x08metadata\x18\x03 \x01(\x0b\x32\x34.qwak.kube.deployment.captain.FeatureSetUserMetadata\x12>\n\x06\x65ntity\x18\x04 \x01(\x0b\x32..qwak.kube.deployment.captain.EntityDefinition\x12\x46\n\x10\x66\x65\x61ture_set_type\x18\x05 \x01(\x0b\x32,.qwak.kube.deployment.captain.FeatureSetType\"R\n\x16\x46\x65\x61tureSetUserMetadata\x12\r\n\x05owner\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x14\n\x0c\x64isplay_name\x18\x03 \x01(\t\"d\n\x10\x45ntityDefinition\x12\x11\n\tentity_id\x18\x01 \x01(\t\x12=\n\x0b\x65ntity_spec\x18\x02 \x01(\x0b\x32(.qwak.kube.deployment.captain.EntitySpec\"p\n\x0e\x46\x65\x61tureSetType\x12R\n\x15streaming_feature_set\x18\x01 \x01(\x0b\x32\x31.qwak.kube.deployment.captain.StreamingFeatureSetH\x00\x42\n\n\x08set_type\"]\n\x13StreamingFeatureSet\x12\x46\n\x0b\x64\x61ta_source\x18\x01 \x01(\x0b\x32\x31.qwak.kube.deployment.captain.StreamingDataSource\"\xe6\x01\n\x0bKafkaSource\x12\x16\n\x0e\x63onsumer_topic\x18\x01 \x01(\t\x12!\n\x19\x63onsumer_bootstrap_server\x18\x02 \x01(\t\x12\x18\n\x10\x63onsumer_timeout\x18\x03 \x01(\t\x12\x16\n\x0e\x63onsumer_group\x18\x04 \x01(\t\x12\x18\n\x10\x61uto_offset_type\x18\x05 \x01(\t\x12\x17\n\x0fsecret_username\x18\x06 \x01(\t\x12\x17\n\x0fsecret_password\x18\x07 \x01(\t\x12\x1e\n\x16\x63onsumer_configuration\x18\x08 \x01(\t\"`\n\x13StreamingDataSource\x12\x41\n\x0ckafka_source\x18\x01 \x01(\x0b\x32).qwak.kube.deployment.captain.KafkaSourceH\x00\x42\x06\n\x04type\"z\n\nEntitySpec\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04keys\x18\x02 \x03(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12;\n\nvalue_type\x18\x04 \x01(\x0e\x32\'.qwak.kube.deployment.captain.ValueType*-\n\tValueType\x12\x0b\n\x07INVALID\x10\x00\x12\n\n\x06STRING\x10\x01\x12\x07\n\x03INT\x10\x02\x42+\n\'com.qwak.ai.kube.deployment.captain.apiP\x01\x62\x06proto3')

_VALUETYPE = DESCRIPTOR.enum_types_by_name['ValueType']
ValueType = enum_type_wrapper.EnumTypeWrapper(_VALUETYPE)
INVALID = 0
STRING = 1
INT = 2


_FEATURESETDEPLOYMENT = DESCRIPTOR.message_types_by_name['FeatureSetDeployment']
_FEATURESETDEPLOYMENT_EXTRADEPLOYMENTCONFIGURATIONENTRY = _FEATURESETDEPLOYMENT.nested_types_by_name['ExtraDeploymentConfigurationEntry']
_FEATURESETDEPLOYMENT_EXTRAENVCONFIGURATIONENTRY = _FEATURESETDEPLOYMENT.nested_types_by_name['ExtraEnvConfigurationEntry']
_BATCHDETAILS = DESCRIPTOR.message_types_by_name['BatchDetails']
_EXECUTIONIDENTIFIER = DESCRIPTOR.message_types_by_name['ExecutionIdentifier']
_FEATURESETDEPLOYMENTSPEC = DESCRIPTOR.message_types_by_name['FeatureSetDeploymentSpec']
_FEATURESETUSERMETADATA = DESCRIPTOR.message_types_by_name['FeatureSetUserMetadata']
_ENTITYDEFINITION = DESCRIPTOR.message_types_by_name['EntityDefinition']
_FEATURESETTYPE = DESCRIPTOR.message_types_by_name['FeatureSetType']
_STREAMINGFEATURESET = DESCRIPTOR.message_types_by_name['StreamingFeatureSet']
_KAFKASOURCE = DESCRIPTOR.message_types_by_name['KafkaSource']
_STREAMINGDATASOURCE = DESCRIPTOR.message_types_by_name['StreamingDataSource']
_ENTITYSPEC = DESCRIPTOR.message_types_by_name['EntitySpec']
FeatureSetDeployment = _reflection.GeneratedProtocolMessageType('FeatureSetDeployment', (_message.Message,), {

  'ExtraDeploymentConfigurationEntry' : _reflection.GeneratedProtocolMessageType('ExtraDeploymentConfigurationEntry', (_message.Message,), {
    'DESCRIPTOR' : _FEATURESETDEPLOYMENT_EXTRADEPLOYMENTCONFIGURATIONENTRY,
    '__module__' : 'qwak.kube_deployment_captain.feature_set_deployment_pb2'
    # @@protoc_insertion_point(class_scope:qwak.kube.deployment.captain.FeatureSetDeployment.ExtraDeploymentConfigurationEntry)
    })
  ,

  'ExtraEnvConfigurationEntry' : _reflection.GeneratedProtocolMessageType('ExtraEnvConfigurationEntry', (_message.Message,), {
    'DESCRIPTOR' : _FEATURESETDEPLOYMENT_EXTRAENVCONFIGURATIONENTRY,
    '__module__' : 'qwak.kube_deployment_captain.feature_set_deployment_pb2'
    # @@protoc_insertion_point(class_scope:qwak.kube.deployment.captain.FeatureSetDeployment.ExtraEnvConfigurationEntry)
    })
  ,
  'DESCRIPTOR' : _FEATURESETDEPLOYMENT,
  '__module__' : 'qwak.kube_deployment_captain.feature_set_deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.kube.deployment.captain.FeatureSetDeployment)
  })
_sym_db.RegisterMessage(FeatureSetDeployment)
_sym_db.RegisterMessage(FeatureSetDeployment.ExtraDeploymentConfigurationEntry)
_sym_db.RegisterMessage(FeatureSetDeployment.ExtraEnvConfigurationEntry)

BatchDetails = _reflection.GeneratedProtocolMessageType('BatchDetails', (_message.Message,), {
  'DESCRIPTOR' : _BATCHDETAILS,
  '__module__' : 'qwak.kube_deployment_captain.feature_set_deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.kube.deployment.captain.BatchDetails)
  })
_sym_db.RegisterMessage(BatchDetails)

ExecutionIdentifier = _reflection.GeneratedProtocolMessageType('ExecutionIdentifier', (_message.Message,), {
  'DESCRIPTOR' : _EXECUTIONIDENTIFIER,
  '__module__' : 'qwak.kube_deployment_captain.feature_set_deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.kube.deployment.captain.ExecutionIdentifier)
  })
_sym_db.RegisterMessage(ExecutionIdentifier)

FeatureSetDeploymentSpec = _reflection.GeneratedProtocolMessageType('FeatureSetDeploymentSpec', (_message.Message,), {
  'DESCRIPTOR' : _FEATURESETDEPLOYMENTSPEC,
  '__module__' : 'qwak.kube_deployment_captain.feature_set_deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.kube.deployment.captain.FeatureSetDeploymentSpec)
  })
_sym_db.RegisterMessage(FeatureSetDeploymentSpec)

FeatureSetUserMetadata = _reflection.GeneratedProtocolMessageType('FeatureSetUserMetadata', (_message.Message,), {
  'DESCRIPTOR' : _FEATURESETUSERMETADATA,
  '__module__' : 'qwak.kube_deployment_captain.feature_set_deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.kube.deployment.captain.FeatureSetUserMetadata)
  })
_sym_db.RegisterMessage(FeatureSetUserMetadata)

EntityDefinition = _reflection.GeneratedProtocolMessageType('EntityDefinition', (_message.Message,), {
  'DESCRIPTOR' : _ENTITYDEFINITION,
  '__module__' : 'qwak.kube_deployment_captain.feature_set_deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.kube.deployment.captain.EntityDefinition)
  })
_sym_db.RegisterMessage(EntityDefinition)

FeatureSetType = _reflection.GeneratedProtocolMessageType('FeatureSetType', (_message.Message,), {
  'DESCRIPTOR' : _FEATURESETTYPE,
  '__module__' : 'qwak.kube_deployment_captain.feature_set_deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.kube.deployment.captain.FeatureSetType)
  })
_sym_db.RegisterMessage(FeatureSetType)

StreamingFeatureSet = _reflection.GeneratedProtocolMessageType('StreamingFeatureSet', (_message.Message,), {
  'DESCRIPTOR' : _STREAMINGFEATURESET,
  '__module__' : 'qwak.kube_deployment_captain.feature_set_deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.kube.deployment.captain.StreamingFeatureSet)
  })
_sym_db.RegisterMessage(StreamingFeatureSet)

KafkaSource = _reflection.GeneratedProtocolMessageType('KafkaSource', (_message.Message,), {
  'DESCRIPTOR' : _KAFKASOURCE,
  '__module__' : 'qwak.kube_deployment_captain.feature_set_deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.kube.deployment.captain.KafkaSource)
  })
_sym_db.RegisterMessage(KafkaSource)

StreamingDataSource = _reflection.GeneratedProtocolMessageType('StreamingDataSource', (_message.Message,), {
  'DESCRIPTOR' : _STREAMINGDATASOURCE,
  '__module__' : 'qwak.kube_deployment_captain.feature_set_deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.kube.deployment.captain.StreamingDataSource)
  })
_sym_db.RegisterMessage(StreamingDataSource)

EntitySpec = _reflection.GeneratedProtocolMessageType('EntitySpec', (_message.Message,), {
  'DESCRIPTOR' : _ENTITYSPEC,
  '__module__' : 'qwak.kube_deployment_captain.feature_set_deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.kube.deployment.captain.EntitySpec)
  })
_sym_db.RegisterMessage(EntitySpec)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\'com.qwak.ai.kube.deployment.captain.apiP\001'
  _FEATURESETDEPLOYMENT_EXTRADEPLOYMENTCONFIGURATIONENTRY._options = None
  _FEATURESETDEPLOYMENT_EXTRADEPLOYMENTCONFIGURATIONENTRY._serialized_options = b'8\001'
  _FEATURESETDEPLOYMENT_EXTRAENVCONFIGURATIONENTRY._options = None
  _FEATURESETDEPLOYMENT_EXTRAENVCONFIGURATIONENTRY._serialized_options = b'8\001'
  _VALUETYPE._serialized_start=2428
  _VALUETYPE._serialized_end=2473
  _FEATURESETDEPLOYMENT._serialized_start=220
  _FEATURESETDEPLOYMENT._serialized_end=1117
  _FEATURESETDEPLOYMENT_EXTRADEPLOYMENTCONFIGURATIONENTRY._serialized_start=973
  _FEATURESETDEPLOYMENT_EXTRADEPLOYMENTCONFIGURATIONENTRY._serialized_end=1040
  _FEATURESETDEPLOYMENT_EXTRAENVCONFIGURATIONENTRY._serialized_start=1042
  _FEATURESETDEPLOYMENT_EXTRAENVCONFIGURATIONENTRY._serialized_end=1102
  _BATCHDETAILS._serialized_start=1120
  _BATCHDETAILS._serialized_end=1268
  _EXECUTIONIDENTIFIER._serialized_start=1270
  _EXECUTIONIDENTIFIER._serialized_end=1313
  _FEATURESETDEPLOYMENTSPEC._serialized_start=1316
  _FEATURESETDEPLOYMENTSPEC._serialized_end=1576
  _FEATURESETUSERMETADATA._serialized_start=1578
  _FEATURESETUSERMETADATA._serialized_end=1660
  _ENTITYDEFINITION._serialized_start=1662
  _ENTITYDEFINITION._serialized_end=1762
  _FEATURESETTYPE._serialized_start=1764
  _FEATURESETTYPE._serialized_end=1876
  _STREAMINGFEATURESET._serialized_start=1878
  _STREAMINGFEATURESET._serialized_end=1971
  _KAFKASOURCE._serialized_start=1974
  _KAFKASOURCE._serialized_end=2204
  _STREAMINGDATASOURCE._serialized_start=2206
  _STREAMINGDATASOURCE._serialized_end=2302
  _ENTITYSPEC._serialized_start=2304
  _ENTITYSPEC._serialized_end=2426
# @@protoc_insertion_point(module_scope)



from collections import defaultdict

import grpc
from _qwak_proto.qwak.builds.builds_pb2 import (
    SUCCESSFUL,
    Build,
    BuildSpec,
    GetBuildResponse,
    ListBuildsResponse,
    RegisterExperimentTrackingValuesResponse,
    RegisterModelSchemaResponse,
    UpdateBuildStatusResponse,
)
from _qwak_proto.qwak.builds.builds_pb2_grpc import BuildsManagementServiceServicer


class BuildsManagementServiceMock(BuildsManagementServiceServicer):
    def __init__(self):
        self.builds = defaultdict(lambda: {})
        self.builds_to_models = defaultdict()
        self.init_status = SUCCESSFUL

    def UpdateBuildStatus(self, request, context):
        model_uuid = self.builds_to_models[request.build_id]
        if model_uuid in self.builds:
            self.builds[model_uuid][request.build_id]["status"] = str(
                request.build_status
            )
        else:
            context.set_code(grpc.StatusCode.NOT_FOUND)
            context.set_details("Received a non existing build id")

        return UpdateBuildStatusResponse()

    def RegisterModelSchema(self, request, context):
        model_uuid = self.builds_to_models[request.build_id]
        if model_uuid in self.builds:
            self.builds[model_uuid][request.build_id]["schema"] = request.model_schema
        return RegisterModelSchemaResponse()

    def RegisterExperimentTrackingValues(self, request, context):
        model_uuid = self.builds_to_models[request.build_id]
        if model_uuid in self.builds:
            spec = BuildSpec(
                build_id=request.build_id,
                experiment_tracking_values=request.experiment_tracking_values,
            )
            self.builds[model_uuid][request.build_id] = {
                "spec": spec,
                "status": 1,
            }
        else:
            context.set_code(grpc.StatusCode.NOT_FOUND)
            context.set_details("Received a non existing build id")
        return RegisterExperimentTrackingValuesResponse()

    def GetBuild(self, request, context):
        model_uuid = self.builds_to_models[request.build_id]
        if model_uuid in self.builds:
            persistent_build = self.builds[model_uuid][request.build_id]

            build = self._persistent_build_to_build_proto(persistent_build)

            return GetBuildResponse(build=build)
        else:
            context.set_code(grpc.StatusCode.NOT_FOUND)
            context.set_details("Received a non existing build id")
            return GetBuildResponse()

    def ListBuilds(self, request, context):
        return ListBuildsResponse(
            builds=[
                self._persistent_build_to_build_proto(persistent_build)
                for persistent_build in self.builds[request.model_uuid].values()
            ]
        )

    @staticmethod
    def _persistent_build_to_build_proto(persistent_build):
        spec = persistent_build["spec"]
        return Build(
            build_spec=BuildSpec(
                build_id=spec.build_id,
                commit_id=spec.commit_id,
                model_id=spec.model_id,
                branch_name=spec.branch_name,
                tags=spec.tags,
                experiment_tracking_values=spec.experiment_tracking_values,
            ),
            build_status=SUCCESSFUL,
            model_schema=persistent_build["schema"]
            if "schema" in persistent_build
            else None,
        )

    def given_build(self, build_id: str, model_uuid: str, build_spec: BuildSpec):
        self.builds[model_uuid] = {
            build_id: {"spec": build_spec, "status": self.init_status}
        }
        self.builds_to_models[build_id] = model_uuid



#
#    Copyright 2024 OpenAI
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ...types import shared_params

__all__ = ["FunctionDefinition"]


class FunctionDefinition(TypedDict, total=False):
    name: Required[str]
    """The name of the function to be called.

    Must be a-z, A-Z, 0-9, or contain underscores and dashes, with a maximum length
    of 64.
    """

    description: str
    """
    A description of what the function does, used by the model to choose when and
    how to call the function.
    """

    parameters: shared_params.FunctionParameters
    """The parameters the functions accepts, described as a JSON Schema object.

    See the
    [guide](https://platform.openai.com/docs/guides/text-generation/function-calling)
    for examples, and the
    [JSON Schema reference](https://json-schema.org/understanding-json-schema/) for
    documentation about the format.

    Omitting `parameters` defines a function with an empty parameter list.
    """



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/feature_store/jobs/job.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!qwak/feature_store/jobs/job.proto\x12\x17qwak.feature.store.jobs\x1a\x1fgoogle/protobuf/timestamp.proto\"\xef\x02\n\tJobRecord\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\x18\n\x10\x66\x65\x61ture_set_name\x18\x02 \x01(\t\x12\x12\n\ntry_number\x18\x03 \x01(\x05\x12\x34\n\tjob_state\x18\x04 \x01(\x0e\x32!.qwak.feature.store.jobs.JobState\x12\x31\n\rcreation_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12.\n\nstart_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08\x65nd_time\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x42\n\x10ingestion_window\x18\x08 \x01(\x0b\x32(.qwak.feature.store.jobs.IngestionWindow\x12\x19\n\x11ingestion_message\x18\t \x01(\t\"\x85\x01\n\x12\x45laborateJobRecord\x12\x36\n\njob_record\x18\x01 \x01(\x0b\x32\".qwak.feature.store.jobs.JobRecord\x12\x16\n\x0estatus_message\x18\x02 \x01(\t\x12\x1f\n\x17\x65rror_technical_details\x18\x03 \x01(\t\"\xe9\x01\n\x0fIngestionWindow\x12<\n\x18ingested_data_start_time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12:\n\x16ingested_data_end_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\"\n\x1aingested_data_updated_rows\x18\x03 \x01(\x05\x12\x38\n\x14\x62\x61tch_execution_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp*w\n\x08JobState\x12\x0b\n\x07INVALID\x10\x00\x12\x0b\n\x07PENDING\x10\x01\x12\x0b\n\x07RUNNING\x10\x02\x12\r\n\tCOMPLETED\x10\x03\x12\n\n\x06\x46\x41ILED\x10\x04\x12\x17\n\x13\x46\x41ILED_CLUSTER_INIT\x10\x05\x12\x10\n\x0c\x43LUSTER_INIT\x10\x06\x42:\n\"com.qwak.ai.feature.store.jobs.apiP\x01Z\x12qwak/fsjobs;fsjobsb\x06proto3')

_JOBSTATE = DESCRIPTOR.enum_types_by_name['JobState']
JobState = enum_type_wrapper.EnumTypeWrapper(_JOBSTATE)
INVALID = 0
PENDING = 1
RUNNING = 2
COMPLETED = 3
FAILED = 4
FAILED_CLUSTER_INIT = 5
CLUSTER_INIT = 6


_JOBRECORD = DESCRIPTOR.message_types_by_name['JobRecord']
_ELABORATEJOBRECORD = DESCRIPTOR.message_types_by_name['ElaborateJobRecord']
_INGESTIONWINDOW = DESCRIPTOR.message_types_by_name['IngestionWindow']
JobRecord = _reflection.GeneratedProtocolMessageType('JobRecord', (_message.Message,), {
  'DESCRIPTOR' : _JOBRECORD,
  '__module__' : 'qwak.feature_store.jobs.job_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.jobs.JobRecord)
  })
_sym_db.RegisterMessage(JobRecord)

ElaborateJobRecord = _reflection.GeneratedProtocolMessageType('ElaborateJobRecord', (_message.Message,), {
  'DESCRIPTOR' : _ELABORATEJOBRECORD,
  '__module__' : 'qwak.feature_store.jobs.job_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.jobs.ElaborateJobRecord)
  })
_sym_db.RegisterMessage(ElaborateJobRecord)

IngestionWindow = _reflection.GeneratedProtocolMessageType('IngestionWindow', (_message.Message,), {
  'DESCRIPTOR' : _INGESTIONWINDOW,
  '__module__' : 'qwak.feature_store.jobs.job_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.jobs.IngestionWindow)
  })
_sym_db.RegisterMessage(IngestionWindow)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\"com.qwak.ai.feature.store.jobs.apiP\001Z\022qwak/fsjobs;fsjobs'
  _JOBSTATE._serialized_start=837
  _JOBSTATE._serialized_end=956
  _JOBRECORD._serialized_start=96
  _JOBRECORD._serialized_end=463
  _ELABORATEJOBRECORD._serialized_start=466
  _ELABORATEJOBRECORD._serialized_end=599
  _INGESTIONWINDOW._serialized_start=602
  _INGESTIONWINDOW._serialized_end=835
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/file_versioning/file_versioning.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*qwak/file_versioning/file_versioning.proto\x12\x1fqwak.management.file_versioning\"\xc7\x01\n\x0b\x46ileTagSpec\x12\x10\n\x08\x62uild_id\x18\x01 \x01(\t\x12\x10\n\x08model_id\x18\x02 \x01(\t\x12\x0b\n\x03tag\x18\x03 \x01(\t\x12I\n\textension\x18\x04 \x01(\x0e\x32\x32.qwak.management.file_versioning.FileExtensionTypeB\x02\x18\x01\x12\x16\n\x0e\x65xtension_type\x18\x05 \x01(\t\x12\x16\n\x0e\x65nvironment_id\x18\x06 \x01(\t\x12\x0c\n\x04path\x18\x07 \x01(\t\"G\n\rFileTagFilter\x12\x16\n\x0ctag_contains\x18\x01 \x01(\tH\x00\x12\x14\n\ntag_prefix\x18\x02 \x01(\tH\x00\x42\x08\n\x06\x66ilter*<\n\x11\x46ileExtensionType\x12\x0b\n\x07INVALID\x10\x00\x12\x07\n\x03PNG\x10\x01\x12\x07\n\x03JPG\x10\x02\x12\x08\n\x04JPEG\x10\x03\x42/\n+com.qwak.ai.management.model_versioning.apiP\x01\x62\x06proto3')

_FILEEXTENSIONTYPE = DESCRIPTOR.enum_types_by_name['FileExtensionType']
FileExtensionType = enum_type_wrapper.EnumTypeWrapper(_FILEEXTENSIONTYPE)
INVALID = 0
PNG = 1
JPG = 2
JPEG = 3


_FILETAGSPEC = DESCRIPTOR.message_types_by_name['FileTagSpec']
_FILETAGFILTER = DESCRIPTOR.message_types_by_name['FileTagFilter']
FileTagSpec = _reflection.GeneratedProtocolMessageType('FileTagSpec', (_message.Message,), {
  'DESCRIPTOR' : _FILETAGSPEC,
  '__module__' : 'qwak.file_versioning.file_versioning_pb2'
  # @@protoc_insertion_point(class_scope:qwak.management.file_versioning.FileTagSpec)
  })
_sym_db.RegisterMessage(FileTagSpec)

FileTagFilter = _reflection.GeneratedProtocolMessageType('FileTagFilter', (_message.Message,), {
  'DESCRIPTOR' : _FILETAGFILTER,
  '__module__' : 'qwak.file_versioning.file_versioning_pb2'
  # @@protoc_insertion_point(class_scope:qwak.management.file_versioning.FileTagFilter)
  })
_sym_db.RegisterMessage(FileTagFilter)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n+com.qwak.ai.management.model_versioning.apiP\001'
  _FILETAGSPEC.fields_by_name['extension']._options = None
  _FILETAGSPEC.fields_by_name['extension']._serialized_options = b'\030\001'
  _FILEEXTENSIONTYPE._serialized_start=354
  _FILEEXTENSIONTYPE._serialized_end=414
  _FILETAGSPEC._serialized_start=80
  _FILETAGSPEC._serialized_end=279
  _FILETAGFILTER._serialized_start=281
  _FILETAGFILTER._serialized_end=352
# @@protoc_insertion_point(module_scope)



from typing import Optional

import grpc
from _qwak_proto.qwak.secret_service.secret_service_pb2 import (
    DeleteSecretRequest,
    GetSecretRequest,
    SetSecretRequest,
)
from _qwak_proto.qwak.secret_service.secret_service_pb2_grpc import SecretServiceStub
from qwak.clients.administration.eco_system.client import EcosystemClient
from qwak.exceptions import QwakException
from qwak.inner.tool.grpc.grpc_tools import create_grpc_channel


class SecretServiceClient:
    """
    gRPC client of the secret service
    """

    def __init__(
        self,
        edge_services_url: Optional[str] = None,
        enable_ssl: bool = True,
        grpc_channel: Optional[grpc.Channel] = None,
        environment_id: Optional[str] = None,
    ):
        if not grpc_channel:
            edge_services_url = self.__get_endpoint_url(
                edge_services_url, environment_id
            )
            grpc_channel = create_grpc_channel(
                url=edge_services_url,
                enable_ssl=enable_ssl,
                status_for_retry=(
                    grpc.StatusCode.UNAVAILABLE,
                    grpc.StatusCode.DEADLINE_EXCEEDED,
                    grpc.StatusCode.INTERNAL,
                ),
                backoff_options={"init_backoff_ms": 250},
            )
        self._secret_service = SecretServiceStub(grpc_channel)

    @staticmethod
    def __get_endpoint_url(endpoint_url=None, environment_id=None):
        if endpoint_url is None:
            user_context = EcosystemClient().get_authenticated_user_context().user
            if environment_id is None:
                environment_id = user_context.environment_details.id

            # If the environment id is not found, try to use the default environment id
            if environment_id not in user_context.account_details.environment_by_id:
                environment_id = user_context.account_details.default_environment_id

            if environment_id not in user_context.account_details.environment_by_id:
                raise QwakException(
                    f"Configuration for environment [{environment_id}] was not found"
                )

            endpoint_url = user_context.account_details.environment_by_id[
                environment_id
            ].configuration.edge_services_url

        return endpoint_url

    def get_secret(self, name) -> str:
        try:
            request = GetSecretRequest(name=name)
            return self._secret_service.GetSecret(request).value
        except grpc.RpcError as e:
            if e.code() == grpc.StatusCode.NOT_FOUND:
                raise QwakException(f"Secret [{name}] not found", status_code=e.code())
            else:
                raise QwakException(
                    f"Failed to get secret, name: [{name}], error code is [{e.code()}], error message is [{e.details()}]"
                )

    def set_secret(self, name, value) -> None:
        try:
            request = SetSecretRequest(name=name, value=value)
            self._secret_service.SetSecret(request)
        except grpc.RpcError as e:
            raise QwakException(
                f"Failed to set secret, name: [{name}], error code is [{e.code()}], error is [{e.details()}]"
            )

    def delete_secret(self, name) -> None:
        try:
            request = DeleteSecretRequest(name=name)
            self._secret_service.DeleteSecret(request)
        except grpc.RpcError as e:
            raise QwakException(
                f"Failed to delete secret, name: [{name}], error is [{e.details()}]"
            )



import grpc
from _qwak_proto.qwak.auto_scaling.v1.auto_scaling_pb2 import AutoScalingConfig
from _qwak_proto.qwak.auto_scaling.v1.auto_scaling_service_pb2 import (
    AttachAutoScalingRequest,
    AttachAutoScalingResponse,
)
from _qwak_proto.qwak.auto_scaling.v1.auto_scaling_service_pb2_grpc import (
    AutoScalingServiceStub,
)
from dependency_injector.wiring import Provide
from qwak.exceptions import QwakException
from qwak.inner.di_configuration import QwakContainer


class AutoScalingClient:
    def __init__(self, grpc_channel=Provide[QwakContainer.core_grpc_channel]):
        self._deployment_management = AutoScalingServiceStub(grpc_channel)

    def attach_autoscaling(
        self, model_id: str, variation_name: str, auto_scaling_config: AutoScalingConfig
    ) -> AttachAutoScalingResponse:
        try:
            return self._deployment_management.AttachAutoScaling(
                AttachAutoScalingRequest(
                    model_id=model_id,
                    variation_name=variation_name,
                    auto_scaling_config=auto_scaling_config,
                )
            )
        except grpc.RpcError as e:
            raise QwakException(f"Failed to attach autoscaling, error is {e.details()}")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/fitness_service/status.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!qwak/fitness_service/status.proto\x12\x14qwak.fitness.service\x1a\x1egoogle/protobuf/wrappers.proto\"\xf3\x01\n\x14RemoteBuildingStatus\x12S\n\x1bremote_building_status_code\x18\x01 \x01(\x0e\x32..qwak.fitness.service.RemoteBuildingStatusCode\x12Q\n\x0f\x66\x61ilure_details\x18\x02 \x01(\x0b\x32\x38.qwak.fitness.service.RemoteBuildingFailureReasonDetails\x12\x33\n\npod_status\x18\x03 \x01(\x0b\x32\x1f.qwak.fitness.service.PodStatus\"\x8c\x01\n\"RemoteBuildingFailureReasonDetails\x12K\n\x0c\x66\x61ilure_code\x18\x01 \x01(\x0e\x32\x35.qwak.fitness.service.RemoteBuildingFailureReasonCode\x12\x19\n\x11technical_details\x18\x02 \x01(\t\"\xa2\x01\n\tPodStatus\x12\x41\n\x12\x63ontainer_statuses\x18\x01 \x03(\x0b\x32%.qwak.fitness.service.ContainerStatus\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\r\n\x05phase\x18\x03 \x01(\t\x12\x0e\n\x06reason\x18\x04 \x01(\t\x12\x12\n\nstart_time\x18\x05 \x01(\t\x12\x0e\n\x06pod_id\x18\x06 \x01(\t\"\x9d\x01\n\x0f\x43ontainerStatus\x12\x14\n\x0c\x63ontainer_id\x18\x01 \x01(\t\x12\r\n\x05image\x18\x02 \x01(\t\x12\x15\n\rrestart_count\x18\x03 \x01(\x05\x12\x0f\n\x07started\x18\x04 \x01(\x08\x12=\n\x0f\x63ontainer_state\x18\x05 \x01(\x0b\x32$.qwak.fitness.service.ContainerState\"\x8f\x02\n\x0e\x43ontainerState\x12N\n\x17\x63ontainer_state_running\x18\x01 \x01(\x0b\x32+.qwak.fitness.service.ContainerStateRunningH\x00\x12T\n\x1a\x63ontainer_state_terminated\x18\x02 \x01(\x0b\x32..qwak.fitness.service.ContainerStateTerminatedH\x00\x12N\n\x17\x63ontainer_state_waiting\x18\x03 \x01(\x0b\x32+.qwak.fitness.service.ContainerStateWaitingH\x00\x42\x07\n\x05state\"+\n\x15\x43ontainerStateRunning\x12\x12\n\nstarted_at\x18\x01 \x01(\t\"\x9d\x01\n\x18\x43ontainerStateTerminated\x12\x14\n\x0c\x63ontainer_id\x18\x01 \x01(\t\x12\x11\n\texit_code\x18\x02 \x01(\x05\x12\x13\n\x0b\x66inished_at\x18\x03 \x01(\t\x12\x0f\n\x07message\x18\x04 \x01(\t\x12\x0e\n\x06reason\x18\x05 \x01(\t\x12\x0e\n\x06signal\x18\x06 \x01(\x05\x12\x12\n\nstarted_at\x18\x07 \x01(\t\"8\n\x15\x43ontainerStateWaiting\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0e\n\x06reason\x18\x02 \x01(\t*\x98\x01\n\x18RemoteBuildingStatusCode\x12\x12\n\x0eINVALID_STATUS\x10\x00\x12\x10\n\x0cINITIALIZING\x10\x01\x12\x0f\n\x0bIN_PROGRESS\x10\x02\x12\x0e\n\nSUCCESSFUL\x10\x03\x12\n\n\x06\x46\x41ILED\x10\x04\x12\r\n\tCANCELLED\x10\x05\x12\r\n\tTIMED_OUT\x10\x06\x12\x0b\n\x07UNKNOWN\x10\x07*\x98\x01\n\x1fRemoteBuildingFailureReasonCode\x12\x10\n\x0cINVALID_CODE\x10\x00\x12\x12\n\x0eUNKNOWN_REASON\x10\x01\x12\x1a\n\x16INSUFFICIENT_RESOURCES\x10\x02\x12\x18\n\x14MISSING_DOCKER_IMAGE\x10\x03\x12\x19\n\x15MEMORY_LIMIT_EXCEEDED\x10\x04\x42#\n\x1f\x63om.qwak.ai.fitness.service.apiP\x01\x62\x06proto3')

_REMOTEBUILDINGSTATUSCODE = DESCRIPTOR.enum_types_by_name['RemoteBuildingStatusCode']
RemoteBuildingStatusCode = enum_type_wrapper.EnumTypeWrapper(_REMOTEBUILDINGSTATUSCODE)
_REMOTEBUILDINGFAILUREREASONCODE = DESCRIPTOR.enum_types_by_name['RemoteBuildingFailureReasonCode']
RemoteBuildingFailureReasonCode = enum_type_wrapper.EnumTypeWrapper(_REMOTEBUILDINGFAILUREREASONCODE)
INVALID_STATUS = 0
INITIALIZING = 1
IN_PROGRESS = 2
SUCCESSFUL = 3
FAILED = 4
CANCELLED = 5
TIMED_OUT = 6
UNKNOWN = 7
INVALID_CODE = 0
UNKNOWN_REASON = 1
INSUFFICIENT_RESOURCES = 2
MISSING_DOCKER_IMAGE = 3
MEMORY_LIMIT_EXCEEDED = 4


_REMOTEBUILDINGSTATUS = DESCRIPTOR.message_types_by_name['RemoteBuildingStatus']
_REMOTEBUILDINGFAILUREREASONDETAILS = DESCRIPTOR.message_types_by_name['RemoteBuildingFailureReasonDetails']
_PODSTATUS = DESCRIPTOR.message_types_by_name['PodStatus']
_CONTAINERSTATUS = DESCRIPTOR.message_types_by_name['ContainerStatus']
_CONTAINERSTATE = DESCRIPTOR.message_types_by_name['ContainerState']
_CONTAINERSTATERUNNING = DESCRIPTOR.message_types_by_name['ContainerStateRunning']
_CONTAINERSTATETERMINATED = DESCRIPTOR.message_types_by_name['ContainerStateTerminated']
_CONTAINERSTATEWAITING = DESCRIPTOR.message_types_by_name['ContainerStateWaiting']
RemoteBuildingStatus = _reflection.GeneratedProtocolMessageType('RemoteBuildingStatus', (_message.Message,), {
  'DESCRIPTOR' : _REMOTEBUILDINGSTATUS,
  '__module__' : 'qwak.fitness_service.status_pb2'
  # @@protoc_insertion_point(class_scope:qwak.fitness.service.RemoteBuildingStatus)
  })
_sym_db.RegisterMessage(RemoteBuildingStatus)

RemoteBuildingFailureReasonDetails = _reflection.GeneratedProtocolMessageType('RemoteBuildingFailureReasonDetails', (_message.Message,), {
  'DESCRIPTOR' : _REMOTEBUILDINGFAILUREREASONDETAILS,
  '__module__' : 'qwak.fitness_service.status_pb2'
  # @@protoc_insertion_point(class_scope:qwak.fitness.service.RemoteBuildingFailureReasonDetails)
  })
_sym_db.RegisterMessage(RemoteBuildingFailureReasonDetails)

PodStatus = _reflection.GeneratedProtocolMessageType('PodStatus', (_message.Message,), {
  'DESCRIPTOR' : _PODSTATUS,
  '__module__' : 'qwak.fitness_service.status_pb2'
  # @@protoc_insertion_point(class_scope:qwak.fitness.service.PodStatus)
  })
_sym_db.RegisterMessage(PodStatus)

ContainerStatus = _reflection.GeneratedProtocolMessageType('ContainerStatus', (_message.Message,), {
  'DESCRIPTOR' : _CONTAINERSTATUS,
  '__module__' : 'qwak.fitness_service.status_pb2'
  # @@protoc_insertion_point(class_scope:qwak.fitness.service.ContainerStatus)
  })
_sym_db.RegisterMessage(ContainerStatus)

ContainerState = _reflection.GeneratedProtocolMessageType('ContainerState', (_message.Message,), {
  'DESCRIPTOR' : _CONTAINERSTATE,
  '__module__' : 'qwak.fitness_service.status_pb2'
  # @@protoc_insertion_point(class_scope:qwak.fitness.service.ContainerState)
  })
_sym_db.RegisterMessage(ContainerState)

ContainerStateRunning = _reflection.GeneratedProtocolMessageType('ContainerStateRunning', (_message.Message,), {
  'DESCRIPTOR' : _CONTAINERSTATERUNNING,
  '__module__' : 'qwak.fitness_service.status_pb2'
  # @@protoc_insertion_point(class_scope:qwak.fitness.service.ContainerStateRunning)
  })
_sym_db.RegisterMessage(ContainerStateRunning)

ContainerStateTerminated = _reflection.GeneratedProtocolMessageType('ContainerStateTerminated', (_message.Message,), {
  'DESCRIPTOR' : _CONTAINERSTATETERMINATED,
  '__module__' : 'qwak.fitness_service.status_pb2'
  # @@protoc_insertion_point(class_scope:qwak.fitness.service.ContainerStateTerminated)
  })
_sym_db.RegisterMessage(ContainerStateTerminated)

ContainerStateWaiting = _reflection.GeneratedProtocolMessageType('ContainerStateWaiting', (_message.Message,), {
  'DESCRIPTOR' : _CONTAINERSTATEWAITING,
  '__module__' : 'qwak.fitness_service.status_pb2'
  # @@protoc_insertion_point(class_scope:qwak.fitness.service.ContainerStateWaiting)
  })
_sym_db.RegisterMessage(ContainerStateWaiting)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\037com.qwak.ai.fitness.service.apiP\001'
  _REMOTEBUILDINGSTATUSCODE._serialized_start=1343
  _REMOTEBUILDINGSTATUSCODE._serialized_end=1495
  _REMOTEBUILDINGFAILUREREASONCODE._serialized_start=1498
  _REMOTEBUILDINGFAILUREREASONCODE._serialized_end=1650
  _REMOTEBUILDINGSTATUS._serialized_start=92
  _REMOTEBUILDINGSTATUS._serialized_end=335
  _REMOTEBUILDINGFAILUREREASONDETAILS._serialized_start=338
  _REMOTEBUILDINGFAILUREREASONDETAILS._serialized_end=478
  _PODSTATUS._serialized_start=481
  _PODSTATUS._serialized_end=643
  _CONTAINERSTATUS._serialized_start=646
  _CONTAINERSTATUS._serialized_end=803
  _CONTAINERSTATE._serialized_start=806
  _CONTAINERSTATE._serialized_end=1077
  _CONTAINERSTATERUNNING._serialized_start=1079
  _CONTAINERSTATERUNNING._serialized_end=1122
  _CONTAINERSTATETERMINATED._serialized_start=1125
  _CONTAINERSTATETERMINATED._serialized_end=1282
  _CONTAINERSTATEWAITING._serialized_start=1284
  _CONTAINERSTATEWAITING._serialized_end=1340
# @@protoc_insertion_point(module_scope)



import gzip
import os
from io import StringIO
from pathlib import Path
from typing import TYPE_CHECKING, Optional

from _qwak_proto.qwak.builds.build_pb2 import (
    CsvFormat,
    DataColumnDefinition,
    DataColumnType,
    DataFormat,
    DataTableDefinition,
)
from _qwak_proto.qwak.builds.build_url_pb2 import BuildVersioningTagsType
from _qwak_proto.qwak.builds.builds_orchestrator_service_pb2 import (
    GetBuildVersioningUploadURLResponse,
)
from qwak.clients.build_orchestrator.client import BuildOrchestratorClient
from qwak.clients.data_versioning.client import DataVersioningManagementClient
from qwak.exceptions import QwakException
from qwak.inner.model_loggers_utils import (
    fetch_build_id,
    upload_data,
    validate_model,
    validate_tag,
)

if TYPE_CHECKING:
    try:
        import pandas as pd
    except ImportError:
        pass


def log_data(
    dataframe: "pd.DataFrame",
    tag: str,
    model_id: Optional[str] = None,
    build_id: Optional[str] = None,
    encoding: str = "utf-8",
) -> None:
    """
    Log data by specific parameters

    Args:
        dataframe: data to log
        tag: tag to save data with
        model_id: optional model id to save data with - if not given found from environment
        build_id: optional build id - if not given found from environment.
        encoding: Encoding for the dataframe - utf-8 is default.
    """
    if os.getenv("QWAK_IS_RUN_LOCAL"):
        return

    if not validate_tag(tag):
        raise QwakException(
            "Tag should contain only letters, numbers, underscore or hyphen"
        )

    data_file_extension = "data.csv.gz"
    data_tag = Path(tag) / data_file_extension
    model_id = validate_model(model_id)

    if not build_id:
        # If called inside a model - build id saved as environment variable
        build_id = fetch_build_id()

    client = BuildOrchestratorClient()
    upload_url_response: GetBuildVersioningUploadURLResponse = (
        client.get_build_versioning_upload_url(
            build_id=build_id,
            model_id=model_id,
            tag=str(data_tag),
            tag_type=BuildVersioningTagsType.DATA_TAG_TYPE,
        )
    )

    string_buffer = StringIO()
    dataframe.to_csv(string_buffer, index=False, escapechar="\\", encoding=encoding)
    upload_data(
        upload_url=upload_url_response.upload_url,
        data=gzip.compress(bytes(string_buffer.getvalue(), "utf-8")),
        content_type="text/plain",
        headers=upload_url_response.headers,
    )

    dataframe_definition = DataTableDefinition(
        columns=[
            DataColumnDefinition(name=str(name), type=map_column_type(str(col_type)))
            for name, col_type in dataframe.dtypes.to_dict().items()
        ],
        data_format=DataFormat(
            csv=CsvFormat(delimiter=",", escape_char="\\", quote_char='"')
        ),
    )

    client.define_build_data_table(
        build_id=build_id, model_id=model_id, tag=tag, table=dataframe_definition
    )

    DataVersioningManagementClient().register_data_tag(
        model_id=model_id, build_id=build_id, tag=tag, extension=data_file_extension
    )


def load_data(
    tag: str,
    model_id: Optional[str] = None,
    build_id: Optional[str] = None,
    encoding: str = "utf-8",
    compressed: bool = True,
) -> "pd.DataFrame":
    """
    Load data by specific parameters

    Args:
        tag: tag to load data from
        model_id: model id to load data with - if not given found from environment
        build_id: optional build id - if not given found from environment.
        encoding: Encoding for the dataframe - utf-8 is default.
        compressed: read a compressed dataframe. Since Qwak version 0.9.39 all logged data frames are compressed by
         default. In order to read a dataframe persisted with an SDK version prior to 0.9.39, add compressed=False.

    Returns:
        wanted dataframe
    """
    if not validate_tag(tag):
        raise QwakException(
            "Tag should contain only letters, numbers, underscore or hyphen"
        )

    file_extension = "csv.gz" if compressed else "csv"
    data_tag = f"{tag}/data.{file_extension}"

    model_id = validate_model(model_id)

    client = BuildOrchestratorClient()
    download_url_response = client.get_build_versioning_download_url(
        build_id=build_id, model_id=model_id, tag=data_tag
    )

    additional_args = {"compression": "gzip"} if compressed else {}
    try:
        import pandas as pd
    except ImportError:
        raise QwakException(
            "Missing Pandas dependency required for logging a dataframe"
        )
    try:
        return pd.read_csv(
            download_url_response.download_url, encoding=encoding, **additional_args
        )
    except Exception as error:
        raise QwakException("Unable to load data: %s" % str(error))


def map_column_type(type_name: str) -> DataColumnType:
    data_type = type_mapper.get(type_name, DataColumnType.INVALID_COLUMN_TYPE)
    if data_type == DataColumnType.INVALID_COLUMN_TYPE:
        raise QwakException(
            f"Failed to map data type {type_name} to Qwak type, which is required for table registration."
            f" Supported types are: {list(type_mapper.keys())}"
        )

    return data_type


type_mapper = {
    "object": DataColumnType.OBJECT,
    "uint8": DataColumnType.INT,
    "int64": DataColumnType.INT,
    "float64": DataColumnType.FLOAT,
    "datetime64": DataColumnType.DATETIME,
    "datetime64[ns]": DataColumnType.DATETIME,
    "datetime64[ns, UTC]": DataColumnType.DATETIME,
    "bool": DataColumnType.BOOLEAN,
}



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2
import qwak.administration.v0.environments.configuration_pb2
import qwak.administration.v0.environments.personalization_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _QwakEnvironmentStatus:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _QwakEnvironmentStatusEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_QwakEnvironmentStatus.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    ENVIRONMENT_STATUS_INVALID: _QwakEnvironmentStatus.ValueType  # 0
    """Default invalid State"""
    ENVIRONMENT_STATUS_ACTIVE: _QwakEnvironmentStatus.ValueType  # 1
    """Marks the environment is currently active for the linked tenant"""
    ENVIRONMENT_STATUS_DISABLED: _QwakEnvironmentStatus.ValueType  # 2
    """Marks that the environment is no longer valid"""

class QwakEnvironmentStatus(_QwakEnvironmentStatus, metaclass=_QwakEnvironmentStatusEnumTypeWrapper): ...

ENVIRONMENT_STATUS_INVALID: QwakEnvironmentStatus.ValueType  # 0
"""Default invalid State"""
ENVIRONMENT_STATUS_ACTIVE: QwakEnvironmentStatus.ValueType  # 1
"""Marks the environment is currently active for the linked tenant"""
ENVIRONMENT_STATUS_DISABLED: QwakEnvironmentStatus.ValueType  # 2
"""Marks that the environment is no longer valid"""
global___QwakEnvironmentStatus = QwakEnvironmentStatus

class _AccountTier:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _AccountTierEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_AccountTier.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    ACCOUNT_TIER_INVALID: _AccountTier.ValueType  # 0
    """Default invalid account tier"""
    FREE: _AccountTier.ValueType  # 1
    """Free account tier"""
    VALIDATED: _AccountTier.ValueType  # 2
    """Validated account tier"""
    PAYING: _AccountTier.ValueType  # 3
    """Paying account tier"""

class AccountTier(_AccountTier, metaclass=_AccountTierEnumTypeWrapper): ...

ACCOUNT_TIER_INVALID: AccountTier.ValueType  # 0
"""Default invalid account tier"""
FREE: AccountTier.ValueType  # 1
"""Free account tier"""
VALIDATED: AccountTier.ValueType  # 2
"""Validated account tier"""
PAYING: AccountTier.ValueType  # 3
"""Paying account tier"""
global___AccountTier = AccountTier

class QwakEnvironment(google.protobuf.message.Message):
    """An environment configuration for a Qwak Tenant.
    Provides a layer of isolation / access control to for a single tenant. Can be used for modeling different lifecycle
    stages, such as - Prod, Dev, and Staging. But can also be used for separating projects and models between teams or
    organizational unit, in a way that every team / OU has its own environment.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    TENANT_ID_FIELD_NUMBER: builtins.int
    ACCOUNT_ID_FIELD_NUMBER: builtins.int
    CLUSTER_ID_FIELD_NUMBER: builtins.int
    ACCOUNT_TIER_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    CONFIGURATION_FIELD_NUMBER: builtins.int
    PERSONALIZATION_FIELD_NUMBER: builtins.int
    CREATED_AT_FIELD_NUMBER: builtins.int
    LAST_MODIFIED_AT_FIELD_NUMBER: builtins.int
    JFROG_TENANT_ID_FIELD_NUMBER: builtins.int
    id: builtins.str
    """Assigned environment ID"""
    name: builtins.str
    """Given environment name"""
    tenant_id: builtins.str
    """Tenant ID which the environment is set up for (migrated to account id)"""
    account_id: builtins.str
    """Account ID which the environment is set up for (migrated to account id)"""
    cluster_id: builtins.str
    """Cluster ID which the environment is located in"""
    account_tier: global___AccountTier.ValueType
    """The account tier of the environment"""
    status: global___QwakEnvironmentStatus.ValueType
    """Current environment status"""
    @property
    def configuration(self) -> qwak.administration.v0.environments.configuration_pb2.QwakEnvironmentConfiguration:
        """Environment configuration"""
    @property
    def personalization(self) -> qwak.administration.v0.environments.personalization_pb2.EnvironmentPersonalization:
        """Environment personalization"""
    @property
    def created_at(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Creation date"""
    @property
    def last_modified_at(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Last modification date"""
    jfrog_tenant_id: builtins.str
    """The Environment's JFrog Tenant ID"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        name: builtins.str = ...,
        tenant_id: builtins.str = ...,
        account_id: builtins.str = ...,
        cluster_id: builtins.str = ...,
        account_tier: global___AccountTier.ValueType = ...,
        status: global___QwakEnvironmentStatus.ValueType = ...,
        configuration: qwak.administration.v0.environments.configuration_pb2.QwakEnvironmentConfiguration | None = ...,
        personalization: qwak.administration.v0.environments.personalization_pb2.EnvironmentPersonalization | None = ...,
        created_at: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        last_modified_at: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        jfrog_tenant_id: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["configuration", b"configuration", "created_at", b"created_at", "jfrog_tenant_id", b"jfrog_tenant_id", "last_modified_at", b"last_modified_at", "optional_jfrog_tenant_id", b"optional_jfrog_tenant_id", "personalization", b"personalization"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["account_id", b"account_id", "account_tier", b"account_tier", "cluster_id", b"cluster_id", "configuration", b"configuration", "created_at", b"created_at", "id", b"id", "jfrog_tenant_id", b"jfrog_tenant_id", "last_modified_at", b"last_modified_at", "name", b"name", "optional_jfrog_tenant_id", b"optional_jfrog_tenant_id", "personalization", b"personalization", "status", b"status", "tenant_id", b"tenant_id"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["optional_jfrog_tenant_id", b"optional_jfrog_tenant_id"]) -> typing_extensions.Literal["jfrog_tenant_id"] | None: ...

global___QwakEnvironment = QwakEnvironment

class EnvironmentApplicationUserCredentials(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    API_KEY_FIELD_NUMBER: builtins.int
    api_key: builtins.str
    """The API key which authenticate as environment application user"""
    def __init__(
        self,
        *,
        api_key: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["api_key", b"api_key"]) -> None: ...

global___EnvironmentApplicationUserCredentials = EnvironmentApplicationUserCredentials



from typing import Dict, Union


class SingletonMeta(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            instance = super().__call__(*args, **kwargs)
            cls._instances[cls] = instance
        return cls._instances[cls]


class QwakExperimentTracking(metaclass=SingletonMeta):
    def __init__(self):
        self.metric = {}
        self.param = {}

    def log_param(self, params: Dict[str, Union[int, str]]):
        self.param = {**self.param, **params}

    def log_params(self, params: Dict[str, Union[int, str]]):
        self.log_param(params)

    def log_metric(self, metrics: Dict[str, Union[int, str]]):
        self.metric = {**self.metric, **metrics}

    def log_metrics(self, metrics: Dict[str, Union[int, str]]):
        self.log_metric(metrics)

    def get_metrics(self):
        return self.metric

    def get_params(self):
        return self.param


def log_metric(metrics):
    client = QwakExperimentTracking()
    client.log_metric(metrics)


def log_param(params):
    client = QwakExperimentTracking()
    client.log_param(params)


def log_metrics(metrics):
    client = QwakExperimentTracking()
    client.log_metric(metrics)


def log_params(params):
    client = QwakExperimentTracking()
    client.log_param(params)


def get_params():
    client = QwakExperimentTracking()
    return client.get_params()


def get_metrics():
    client = QwakExperimentTracking()
    return client.get_metrics()



from qwak.feature_store.data_sources.batch.athena import AthenaSource
from qwak.feature_store.data_sources.batch.big_query import BigQuerySource
from qwak.feature_store.data_sources.batch.clickhouse import ClickhouseSource
from qwak.feature_store.data_sources.batch.csv import CsvSource
from qwak.feature_store.data_sources.batch.elastic_search import ElasticSearchSource
from qwak.feature_store.data_sources.batch.filesystem.aws import (
    AnonymousS3Configuration,
    AwsS3AssumeRoleFileSystemConfiguration,
    AwsS3FileSystemConfiguration,
)
from qwak.feature_store.data_sources.batch.filesystem.gcp import (
    GcpGcsServiceAccountImpersonation,
    GcpGcsUnauthenticated,
)
from qwak.feature_store.data_sources.batch.mongodb import MongoDbSource
from qwak.feature_store.data_sources.batch.mysql import MysqlSource
from qwak.feature_store.data_sources.batch.parquet import ParquetSource
from qwak.feature_store.data_sources.batch.postgres import PostgresSource
from qwak.feature_store.data_sources.batch.redshift import RedshiftSource
from qwak.feature_store.data_sources.batch.snowflake import SnowflakeSource
from qwak.feature_store.data_sources.batch.vertica import VerticaSource
from qwak.feature_store.data_sources.streaming.kafka import KafkaSource
from qwak.feature_store.data_sources.streaming.kafka.authentication import (
    PlainAuthentication,
    SaslAuthentication,
    SaslMechanism,
    SecurityProtocol,
    SslAuthentication,
)
from qwak.feature_store.data_sources.streaming.kafka.deserialization import (
    CustomDeserializer,
    Deserializer,
    GenericDeserializer,
    MessageFormat,
)

__all__ = [
    "AthenaSource",
    "BigQuerySource",
    "ClickhouseSource",
    "CsvSource",
    "ElasticSearchSource",
    "AwsS3FileSystemConfiguration",
    "AnonymousS3Configuration",
    "AwsS3AssumeRoleFileSystemConfiguration",
    "GcpGcsServiceAccountImpersonation",
    "GcpGcsUnauthenticated",
    "MongoDbSource",
    "MysqlSource",
    "ParquetSource",
    "PostgresSource",
    "RedshiftSource",
    "SnowflakeSource",
    "VerticaSource",
    "KafkaSource",
    "Deserializer",
    "CustomDeserializer",
    "GenericDeserializer",
    "MessageFormat",
    "PlainAuthentication",
    "SslAuthentication",
    "SaslMechanism",
    "SaslAuthentication",
    "SecurityProtocol",
]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/ecosystem/v0/credentials.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#qwak/ecosystem/v0/credentials.proto\x12\x11qwak.ecosystem.v0\x1a\x1fgoogle/protobuf/timestamp.proto\"\xc3\x01\n\x10\x43loudCredentials\x12O\n\x19\x61ws_temporary_credentials\x18\x01 \x01(\x0b\x32*.qwak.ecosystem.v0.AwsTemporaryCredentialsH\x00\x12O\n\x19gcp_temporary_credentials\x18\x02 \x01(\x0b\x32*.qwak.ecosystem.v0.GcpTemporaryCredentialsH\x00\x42\r\n\x0b\x63redentials\"\xa7\x01\n\x17\x41wsTemporaryCredentials\x12\x15\n\raccess_key_id\x18\x01 \x01(\t\x12\x19\n\x11secret_access_key\x18\x02 \x01(\t\x12\x15\n\rsession_token\x18\x03 \x01(\t\x12\x33\n\x0f\x65xpiration_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0e\n\x06region\x18\x05 \x01(\t\"d\n\x17GcpTemporaryCredentials\x12\x14\n\x0c\x61\x63\x63\x65ss_token\x18\x01 \x01(\t\x12\x33\n\x0f\x65xpiration_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\xb7\x01\n\x1c\x63om.qwak.ai.common.ecosystemP\x01Z\x94\x01github.com/qwak-ai/qwak-platform/services/shared/java/libs/qwak-common/qwak-common-api/qwak-platform-ecosystem-api/pb/qwak/ecosystem/v0;ecosystem_v0b\x06proto3')



_CLOUDCREDENTIALS = DESCRIPTOR.message_types_by_name['CloudCredentials']
_AWSTEMPORARYCREDENTIALS = DESCRIPTOR.message_types_by_name['AwsTemporaryCredentials']
_GCPTEMPORARYCREDENTIALS = DESCRIPTOR.message_types_by_name['GcpTemporaryCredentials']
CloudCredentials = _reflection.GeneratedProtocolMessageType('CloudCredentials', (_message.Message,), {
  'DESCRIPTOR' : _CLOUDCREDENTIALS,
  '__module__' : 'qwak.ecosystem.v0.credentials_pb2'
  # @@protoc_insertion_point(class_scope:qwak.ecosystem.v0.CloudCredentials)
  })
_sym_db.RegisterMessage(CloudCredentials)

AwsTemporaryCredentials = _reflection.GeneratedProtocolMessageType('AwsTemporaryCredentials', (_message.Message,), {
  'DESCRIPTOR' : _AWSTEMPORARYCREDENTIALS,
  '__module__' : 'qwak.ecosystem.v0.credentials_pb2'
  # @@protoc_insertion_point(class_scope:qwak.ecosystem.v0.AwsTemporaryCredentials)
  })
_sym_db.RegisterMessage(AwsTemporaryCredentials)

GcpTemporaryCredentials = _reflection.GeneratedProtocolMessageType('GcpTemporaryCredentials', (_message.Message,), {
  'DESCRIPTOR' : _GCPTEMPORARYCREDENTIALS,
  '__module__' : 'qwak.ecosystem.v0.credentials_pb2'
  # @@protoc_insertion_point(class_scope:qwak.ecosystem.v0.GcpTemporaryCredentials)
  })
_sym_db.RegisterMessage(GcpTemporaryCredentials)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\034com.qwak.ai.common.ecosystemP\001Z\224\001github.com/qwak-ai/qwak-platform/services/shared/java/libs/qwak-common/qwak-common-api/qwak-platform-ecosystem-api/pb/qwak/ecosystem/v0;ecosystem_v0'
  _CLOUDCREDENTIALS._serialized_start=92
  _CLOUDCREDENTIALS._serialized_end=287
  _AWSTEMPORARYCREDENTIALS._serialized_start=290
  _AWSTEMPORARYCREDENTIALS._serialized_end=457
  _GCPTEMPORARYCREDENTIALS._serialized_start=459
  _GCPTEMPORARYCREDENTIALS._serialized_end=559
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class GetDataSourceSourceCodeDownloadURLRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_SOURCE_NAME_FIELD_NUMBER: builtins.int
    data_source_name: builtins.str
    def __init__(
        self,
        *,
        data_source_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["data_source_name", b"data_source_name"]) -> None: ...

global___GetDataSourceSourceCodeDownloadURLRequest = GetDataSourceSourceCodeDownloadURLRequest

class GetDataSourceSourceCodeDownloadURLResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DOWNLOAD_URL_FIELD_NUMBER: builtins.int
    download_url: builtins.str
    def __init__(
        self,
        *,
        download_url: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["download_url", b"download_url"]) -> None: ...

global___GetDataSourceSourceCodeDownloadURLResponse = GetDataSourceSourceCodeDownloadURLResponse

class GetDataSourceSourceCodeUploadRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_SOURCE_NAME_FIELD_NUMBER: builtins.int
    REPOSITORY_NAME_FIELD_NUMBER: builtins.int
    data_source_name: builtins.str
    repository_name: builtins.str
    def __init__(
        self,
        *,
        data_source_name: builtins.str = ...,
        repository_name: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["repository_details", b"repository_details", "repository_name", b"repository_name"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["data_source_name", b"data_source_name", "repository_details", b"repository_details", "repository_name", b"repository_name"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["repository_details", b"repository_details"]) -> typing_extensions.Literal["repository_name"] | None: ...

global___GetDataSourceSourceCodeUploadRequest = GetDataSourceSourceCodeUploadRequest

class GetDataSourceSourceCodeUploadResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class ExtraHeadersEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    UPLOAD_URL_FIELD_NUMBER: builtins.int
    EXTRA_HEADERS_FIELD_NUMBER: builtins.int
    upload_url: builtins.str
    """Upload url intended for the data source source code"""
    @property
    def extra_headers(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """Extra headers that are required for the upload"""
    def __init__(
        self,
        *,
        upload_url: builtins.str = ...,
        extra_headers: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["extra_headers", b"extra_headers", "upload_url", b"upload_url"]) -> None: ...

global___GetDataSourceSourceCodeUploadResponse = GetDataSourceSourceCodeUploadResponse



import sys
from abc import ABC, abstractmethod
from dataclasses import dataclass, field
from typing import TYPE_CHECKING, Callable, Dict, List, Optional, Tuple, Any
from warnings import warn

if TYPE_CHECKING:
    import pyspark.pandas as psd
    import pyspark.sql as psql

import importlib

from _qwak_proto.qwak.feature_store.features.aggregation_pb2 import (
    AggregationField as ProtoAggregationField,
    TimeWindow as ProtoTimeWindow,
)
from _qwak_proto.qwak.feature_store.features.feature_set_types_pb2 import (
    KoalasTransformation as ProtoKoalasTransformation,
    PandasOnSparkTransformation as ProtoPandasOnSparkTransformation,
    PySparkTransformation as ProtoPySparkTransformation,
    SqlTransformation as ProtoSqlTransformation,
    TransformArguments as ProtoTransformArguments,
    Transformation as ProtoTransformation,
    UdfTransformation as ProtoUdfTransformation,
)
from qwak.exceptions import QwakException
from qwak.feature_store._common.value import (
    UPDATE_QWAK_SDK_WITH_FEATURE_STORE_EXTRA_MSG,
    missing_pyspark_exception_message,
)
from qwak.feature_store.feature_sets.transformations.aggregations.aggregations import (
    BaseAggregate,
    QwakAggregation,
)
from qwak.feature_store.feature_sets.transformations.aggregations.windows import Window
from qwak.feature_store.feature_sets.transformations.validations.validations_util import (
    validate_function,
    validate_qwargs,
)
from qwak.tools.logger import logger as qwak_logger

logger = qwak_logger.get_qwak_logger()


@dataclass
class BaseTransformation(ABC):
    """
    Base class for Qwak transformations.

    Windows and aggregates are supported only for streaming aggregation featuresets.
    """

    windows: List[Window] = field(init=False, default_factory=list)
    aggregations: Tuple[BaseAggregate] = field(init=False, default_factory=tuple)
    _artifact_path: Optional[str] = field(init=False, default=None)

    def aggregate(self, *aggregations: QwakAggregation):
        self.aggregations += aggregations
        return self

    def by_windows(self, *time_windows: str):
        self.windows += Window._from_string(*set(time_windows))
        return self

    def get_names(self) -> List[str]:
        return [a.get_name() for a in self.aggregations]

    def get_features_names(self) -> List[str]:
        if self.windows and self.aggregations:
            return [
                f"{name}_{tu.get_feature_suffix()}"
                for tu in self.windows
                for name in self.get_names()
            ]
        else:
            raise QwakException(
                "Feature names are retrieved for streaming aggregation featuresets only"
            )

    def _get_aggregations_proto(self) -> List[ProtoAggregationField]:
        """
        Converts a user-defined streaming aggregation transformation to a list of proto aggregation fields
        Time windows are applied to all defined aggregations (cartesian product)
        """

        time_windows: List[ProtoTimeWindow] = [
            ProtoTimeWindow(length=window.length, time_unit=window.time_unit_proto)
            for window in self.windows
        ]

        proto_aggregations: List[ProtoAggregationField] = []

        for aggregation in self.aggregations:
            aggregation_params = {
                aggregation._key: aggregation.to_proto(),
                "time_windows": time_windows,
            }
            if aggregation.has_alias():
                aggregation_params["field_name_prefix"] = aggregation._alias_name

            proto_aggregations.append(ProtoAggregationField(**aggregation_params))

        return proto_aggregations

    @classmethod
    def _from_proto(cls, proto: ProtoTransformation):
        function_mapping = {
            "sql_transformation": SparkSqlTransformation,
            "koalas_transformation": KoalasTransformation,
            "pyspark_transformation": PySparkTransformation,
            "udf_transformation": UdfTransformation,
            "pandas_on_spark_transformation": PandasOnSparkTransformation,
        }

        function_type: str = proto.WhichOneof("type")

        if function_type in function_mapping:
            function_class = function_mapping.get(function_type)
            return function_class._from_proto(proto)

        raise QwakException(f"Got unsupported transformation type: {function_type}")

    @abstractmethod
    def _to_proto(self, artifact_path: Optional[str] = None) -> ProtoTransformation:
        pass

    def get_functions(self) -> Optional[List[Callable]]:
        return None

    def _validate_udfs(self):
        udfs: Optional[List[Callable]] = self.get_functions()

        if udfs and len(udfs) > 0:
            python_major = sys.version_info[0]
            python_minor = sys.version_info[1]
            if f"{python_major}.{python_minor}" != "3.8":
                raise QwakException(
                    f"Feature store UDFs are only supported for python 3.8, instead got: "
                    f"{python_major}.{python_minor}"
                )


@dataclass
class KoalasTransformation(BaseTransformation):
    """
    Koalas transformation, providing the user with the ability to define a Koalas based UDF for the transformation
    of the FeatureSet. This option will be deprecated in future versions.
    @param function: The Koalas function defined for the transformation
    @type: Callable
    @deprecated
    """

    function: Optional[Callable] = None
    qwargs: Optional[Dict[str, str]] = None

    def __post_init__(self):
        warn(
            "Koalas transformation is about to be deprecated. "
            "Please use PySparkTransformation, SparkSqlTransformation or UDFTransformation instead",
            DeprecationWarning,
            stacklevel=2,
        )

        self.qwargs = self.qwargs if self.qwargs else {}
        self.qwargs = {str(k): str(v) for k, v in self.qwargs.items()}
        self._artifact_path = None

        if not self.function:
            raise QwakException(
                "Please provide a valid function for the koalas transformation"
            )

    def _to_proto(self, artifact_path: Optional[str] = None) -> ProtoTransformation:
        self._validate_udfs()

        if artifact_path:
            self._artifact_path = artifact_path

        return ProtoTransformation(
            koalas_transformation=ProtoKoalasTransformation(
                function_name=self.function.__name__,
                qwargs=ProtoTransformArguments(qwargs=self.qwargs),
            ),
            artifact_path=self._artifact_path,
        )

    @classmethod
    def _from_proto(cls, proto: "ProtoTransformation"):
        koalas_transformation = proto.koalas_transformation
        qwargs = {}
        if koalas_transformation.WhichOneof("args_option") == "qwargs":
            qwargs = koalas_transformation.qwargs.qwargs

        def f():
            print(
                f"Loading Koalas UDFs is not supported. Can not load {koalas_transformation.function_name}"
            )

        f.__name__ = koalas_transformation.function_name

        return cls(function=f, qwargs=qwargs)

    def get_functions(self) -> Optional[List[Callable]]:
        return [self.function]


@dataclass
class PySparkTransformation(BaseTransformation):
    """
    PySpark transformation, providing the user with the ability to define a PySpark based UDF for the transformation
    of the FeatureSet.
    @param function: The PySpark function defined for the transformation
    @type: Callable
    @deprecated
    """

    function: Callable[
        [Dict[str, "psql.DataFrame"], Dict[str, Any]],
        "psql.DataFrame",
    ]
    qwargs: Optional[Dict[str, str]] = None

    def __post_init__(self):
        self.qwargs = self.qwargs if self.qwargs else {}
        self._artifact_path = None

        if importlib.util.find_spec("pyspark"):
            import pyspark

            validate_function(self.function, pyspark.sql.DataFrame, self.__class__)
        else:
            raise QwakException(
                missing_pyspark_exception_message(self.__class__.__name__)
            )

        validate_qwargs(self.qwargs)

    def _to_proto(self, artifact_path: Optional[str] = None) -> ProtoTransformation:
        self._validate_udfs()

        if artifact_path:
            self._artifact_path = artifact_path

        return ProtoTransformation(
            pyspark_transformation=ProtoPySparkTransformation(
                function_name=self.function.__name__,
                qwargs=ProtoTransformArguments(qwargs=self.qwargs),
            ),
            artifact_path=self._artifact_path,
        )

    @classmethod
    def _from_proto(cls, proto: "ProtoTransformation"):
        try:
            import pyspark.sql as ps
        except Exception:
            raise QwakException(missing_pyspark_exception_message(cls.__name__))

        pyspark_transformation = proto.pyspark_transformation
        qwargs = {}
        if pyspark_transformation.WhichOneof("args_option") == "qwargs":
            qwargs = pyspark_transformation.qwargs.qwargs

        def f(dfs: Dict[str, ps.DataFrame]) -> ps.DataFrame:
            print(
                f"Loading Pyspark UDFs is not supported. Can not load {pyspark_transformation.function_name}"
            )

        f.__name__ = pyspark_transformation.function_name

        return cls(function=f, qwargs=qwargs)

    def get_functions(self) -> Optional[List[Callable]]:
        return [self.function]


@dataclass
class SparkSqlTransformation(BaseTransformation):
    """
    A Spark SQL transformation
    :param sql: A valid Spark SQL transformation
    :param functions: PySpark Pandas UDFs
    Example transformation:
    ... code-block:: python
        SparkSqlTransformation("SELECT user_id, age FROM data_source")
    Example transformation with additional UDFs:
    ... code-block:: python
        @qwak_pandas_udf(output_schema=Schema([
            Column(type=Type.long)]))
        def plus_one(column_a: pd.Series) -> pd.Series:
            return column_a + 1


        @qwak_pandas_udf(output_schema=Schema([
            Column(type=Type.long)]))
        def mul_by_two(column_a: pd.Series) -> pd.Series:
            return column_a * 2

        SparkSqlTransformation("SELECT user_id, age FROM data_source", functions=[plus_one, mul_by_two])
    """

    sql: str = str()
    functions: Optional[List[Callable]] = None

    def __post_init__(self):
        if not self.sql:
            raise QwakException(
                "SQL statement can not be empty when using a SparkSqlTransformation"
            )

    @classmethod
    def _from_proto(cls, proto: "ProtoSqlTransformation"):
        return cls(sql=proto.sql_transformation.sql)

    def _to_proto(self, artifact_path: Optional[str] = None) -> ProtoTransformation:
        self._validate_udfs()

        return ProtoTransformation(
            sql_transformation=ProtoSqlTransformation(
                sql=self.sql,
                function_names=(
                    [function.__name__ for function in self.functions]
                    if self.functions
                    else None
                ),
            ),
            artifact_path=artifact_path,
        )

    def get_functions(self) -> Optional[List[Callable]]:
        return self.functions


@dataclass
class UdfTransformation(BaseTransformation):
    """
    A UDF Transformation
    :param function: A valid user defined pandas function decorated with the @qwak_pandas_udf decorator
    Example transformation:
    ... code-block:: python

    @qwak_pandas_udf(output_schema=Schema([Column(type=Type.long)]))
    def my_pandas_udf() -> pd.Series:
        data = np.random.randn(3)
        series = pd.Series(data)
        return series

    UdfTransformation(function=my_pandas_udf)
    """

    function: Callable = ()

    def __post_init__(self):
        if not self.function:
            raise QwakException("Please provide a function to the UdfTransformation")

    def _to_proto(self, artifact_path: Optional[str] = None) -> ProtoTransformation:
        self._validate_udfs()

        if artifact_path:
            self._artifact_path = artifact_path

        return ProtoTransformation(
            artifact_path=artifact_path,
            udf_transformation=ProtoUdfTransformation(
                function_name=self.function.__name__
            ),
        )

    @classmethod
    def _from_proto(cls, proto: "ProtoTransformation"):
        return cls(
            function=lambda x: print("Loading UDFs is not yet supported"),
        )

    def get_functions(self) -> Optional[List[Callable]]:
        return [self.function]


@dataclass
class PandasOnSparkTransformation(BaseTransformation):
    """
    Pandas On Spark transformation.
    This class provides the user with the ability to define a Pandas On Spark based UDF as the transformation of the Feature Set.

    Attributes
    ----------
    function : Callable
        The Pandas On Spark function defined for the transformation.
    qwargs : dict, optional
        Runtime parameters (e.g., qwak_ingestion_start_timestamp, qwak_ingestion_end_timestamp).
    """

    function: Callable[
        [Dict[str, "psd.DataFrame"], Dict[str, Any]],
        "psd.DataFrame",
    ]
    qwargs: Optional[Dict[str, str]] = None

    def __post_init__(self):
        self.qwargs = self.qwargs if self.qwargs else {}
        self._artifact_path = None
        if importlib.util.find_spec("pyspark"):
            import pyspark.pandas as psd

            validate_function(self.function, psd.DataFrame, self.__class__)
        else:
            raise QwakException(
                missing_pyspark_exception_message(self.__class__.__name__)
            )
        validate_qwargs(self.qwargs)

    def _to_proto(self, artifact_path: Optional[str] = None) -> ProtoTransformation:
        self._validate_udfs()

        if artifact_path:
            self._artifact_path = artifact_path

        return ProtoTransformation(
            pandas_on_spark_transformation=ProtoPandasOnSparkTransformation(
                function_name=self.function.__name__,
                qwargs=ProtoTransformArguments(qwargs=self.qwargs),
            ),
            artifact_path=self._artifact_path,
        )

    @classmethod
    def _from_proto(cls, proto: "ProtoTransformation"):
        try:
            import pyspark.pandas as ps
        except Exception:
            raise QwakException(
                "Missing 'pyspark' dependency required for PandasOnSpark transformation. "
                f"{UPDATE_QWAK_SDK_WITH_FEATURE_STORE_EXTRA_MSG}"
            )
        pandas_on_spark_transformation = proto.pandas_on_spark_transformation
        qwargs = {}
        if pandas_on_spark_transformation.WhichOneof("args_option") == "qwargs":
            qwargs = pandas_on_spark_transformation.qwargs.qwargs

        def f(dfs: Dict[str, ps.DataFrame]) -> ps.DataFrame:
            print(
                f"Loading Pandas On Spark UDFs is not supported. Can not load {pandas_on_spark_transformation.function_name}"
            )

        f.__name__ = pandas_on_spark_transformation.function_name

        return cls(function=f, qwargs=qwargs)

    def get_functions(self) -> Optional[List[Callable]]:
        return [self.function]
